/******************************************************************************
 * Copyright 2020 The siasun Transport Authors. All Rights Reserved.
 * Author: limengqi
 * Licensed under the Apache License, Version 1.0 (the "License");
 *****************************************************************************/
#include "messagecoder.h"
#include"candatatype.h"
#include"string.h"
#include"../common/canmsgobdmsgid.h"

namespace  robot{
namespace canbusSpace {

#define CAN_MSG_LEN 8

//字符串反转
void MessageCoder::CharReverse(unsigned char * s, int n)
{
    for (int i = 0,j=n-1; i < j; i++,j--)
    {
        char c = s[i];
        s[i] = s[j];
        s[j] = c;
    }
}

//执行机构使能请求--数据解析
void MessageCoder::CanMsgAnalyseID_7F5(unsigned char * ucData, T_ACTUATOR_ENABLE_REQUEST *tActEnableReq)
{
    T_CONTROL_CMD_7F5 tCtrCmd7F5 = { 0 };
//    unsigned char ucCanMsgSrc[8] = { 0 };
//    memcpy(ucCanMsgSrc, ucData, CAN_MSG_LEN);
    CharReverse(ucData,8);
    memcpy(&tCtrCmd7F5, ucData, CAN_MSG_LEN);
    tActEnableReq->uiAccClchCtrReq = tCtrCmd7F5.uiAccClchCtrReq;
    tActEnableReq->uiAccMotEnReq = tCtrCmd7F5.uiAccMotEnReq;
    tActEnableReq->uiBrkMotEnReq = tCtrCmd7F5.uiBrkMotEnReq;
    tActEnableReq->uiClchMotEnReq = tCtrCmd7F5.uiClchMotEnReq;
    tActEnableReq->uiStrMotEnReq = tCtrCmd7F5.uiStrMotEnReq;
    tActEnableReq->uiXSftMotEnReq = tCtrCmd7F5.uiXSftMotEnReq;
    tActEnableReq->uiYSftMotEnReq = tCtrCmd7F5.uiYSftMotEnReq;
}

//转向执行机构控制命令--数据解析
void MessageCoder::CanMsgAnalyseID_5F4(unsigned char * ucData, T_STR_ACT_CTR_REQ * tStrActCtrReq)
{
    T_CONTROL_CMD_5F4 tCtrCmd5F4 = { 0 };
    T_CONTROL_CMD_5F4_UNION tCtrCmd5F4Union;

//    unsigned char ucCanMsgSrc[8] = { 0 };
//    memcpy(ucCanMsgSrc, ucData, CAN_MSG_LEN);
    CharReverse(ucData,8);
    memcpy(&tCtrCmd5F4, ucData, CAN_MSG_LEN);

    memset(&tCtrCmd5F4Union, 0, sizeof(T_CONTROL_CMD_5F4_UNION));
    tCtrCmd5F4Union.tCtrCmd5F4Src.iStrSpdCtrReqL = tCtrCmd5F4.iStrSpdCtrReqL;
    tCtrCmd5F4Union.tCtrCmd5F4Src.iStrSpdCtrReqH = tCtrCmd5F4.iStrSpdCtrReqH;

    tStrActCtrReq->iStrAngCtrReq = tCtrCmd5F4.iStrAngCtrReq;
    tStrActCtrReq->uiStrCtrMdReq = tCtrCmd5F4.uiStrCtrMdReq;
    tStrActCtrReq->iStrSpdCtrReq = tCtrCmd5F4Union.tCtrCmd5F4Dest.iStrSpdCtrReq;
    tStrActCtrReq->iStrTrqCtrReq = tCtrCmd5F4.iStrTrqCtrReq;
}

//车辆状态1--数据解析
void MessageCoder::CanMsgAnalyseID_6F1(unsigned char * ucData, T_VEHICLE_STATUS_1 * tVehicleStat1)
{
    T_STATE_IN_6F1 tStatIn6F1 = { 0 };
//    unsigned char ucCanMsgSrc[8] = { 0 };

//    memcpy(ucCanMsgSrc, ucData, CAN_MSG_LEN);
    CharReverse(ucData,8);
    memcpy(&tStatIn6F1, ucData, CAN_MSG_LEN);

    tVehicleStat1->uiAccPsng = tStatIn6F1.uiAccPsng;
    tVehicleStat1->uiBrkPsng = tStatIn6F1.uiBrkPsng;
    tVehicleStat1->uiClchPsng = tStatIn6F1.uiClchPsng;
    tVehicleStat1->uiDrvSftTrgtGear = tStatIn6F1.uiDrvSftTrgtGear;
    tVehicleStat1->uiVehSftMd = tStatIn6F1.uiVehSftMd;
    tVehicleStat1->uiSftPsng = tStatIn6F1.uiSftPsng;
    tVehicleStat1->uiVehHBrkStat = tStatIn6F1.uiVehHBrkStat;
    tVehicleStat1->uiVehSwStat = tStatIn6F1.uiVehSwStat;
}

//车辆状态2--数据解析
void MessageCoder::CanMsgAnalyseID_6F2(unsigned char * ucData, T_VEHICLE_STATUS_2 * tVehicleStat2)
{
    T_STATE_IN_6F2 tStatIn6F2 = { 0 };
    T_STATE_IN_6F2_UNION tStatIn6F2Union;
//    unsigned char ucCanMsgSrc[8] = { 0 };

//    memcpy(ucCanMsgSrc, ucData, CAN_MSG_LEN);
    CharReverse(ucData,8);
    memcpy(&tStatIn6F2, ucData, CAN_MSG_LEN);

    memset(&tStatIn6F2Union, 0, sizeof(T_STATE_IN_6F2_UNION));
    tStatIn6F2Union.tStateIn6F2Src.iStrAngGrdL = tStatIn6F2.iStrAngGrdL;
    tStatIn6F2Union.tStateIn6F2Src.iStrAngGrdH = tStatIn6F2.iStrAngGrdH;

    tVehicleStat2->iStrAng = tStatIn6F2.iStrAng;
    tVehicleStat2->iStrAngGrd = tStatIn6F2Union.tStateIn6F2Dest.iStrAngGrd;
    tVehicleStat2->uiVehSpd = tStatIn6F2.uiVehSpd;
    tVehicleStat2->uiEngSpd = tStatIn6F2.uiEngSpd;
}

//车辆状态3--数据解析
void MessageCoder::CanMsgAnalyseID_6F3(unsigned char * ucData, T_VEHICLE_STATUS_3 * tVehicleStat3)
{
    T_STATE_IN_6F3 tStatIn6F3 = { 0 };
//    unsigned char ucCanMsgSrc[8] = { 0 };

//    memcpy(ucCanMsgSrc, ucData, CAN_MSG_LEN);
    CharReverse(ucData,8);
    memcpy(&tStatIn6F3, ucData, CAN_MSG_LEN);

    tVehicleStat3->uiVehBatQut = tStatIn6F3.uiVehBatQut;
    tVehicleStat3->uiVehBatCur = tStatIn6F3.uiVehBatCur;
    tVehicleStat3->uiVehBatVlt = tStatIn6F3.uiVehBatVlt;
}

//工控机状态--数据解析
void MessageCoder::CanMsgAnalyseID_6F4(unsigned char * ucData, T_COMPUTER_STATUS * tComputerStat)
{
    T_STATE_IN_6F4 tStatIn6F4 = { 0 };
//    unsigned char ucCanMsgSrc[8] = { 0 };

//    memcpy(ucCanMsgSrc, ucData, CAN_MSG_LEN);
    CharReverse(ucData,8);
    memcpy(&tStatIn6F4, ucData, CAN_MSG_LEN);

    tComputerStat->uiCmpStat = tStatIn6F4.uiCmpStat;
}

//档位 Z 轴状态反馈--数据解析
void MessageCoder::CanMsgAnalyseID_6F5(unsigned char * ucData, T_ZSFT_STATUS * tZSftStat)
{
    T_STATE_OUT_6F5 tStatOut6F5 = { 0 };
//    unsigned char ucCanMsgSrc[8] = { 0 };

//    memcpy(ucCanMsgSrc, ucData, CAN_MSG_LEN);
    CharReverse(ucData,8);
    memcpy(&tStatOut6F5, ucData, CAN_MSG_LEN);

    tZSftStat->iZSftMotAng = tStatOut6F5.iZSftMotAng;
    tZSftStat->iZSftMotSpd = tStatOut6F5.iZSftMotSpd;
    tZSftStat->iZSftMotTrq = tStatOut6F5.iZSftMotTrq;
    tZSftStat->uiZSftArmCtrAlive = tStatOut6F5.uiZSftArmCtrAlive;
    tZSftStat->uiZSftMotClbr = tStatOut6F5.uiZSftMotClbr;
    tZSftStat->uiZSftMotCur = tStatOut6F5.uiZSftMotCur;
}

//机器人执行机构实际位置--数据解析
void MessageCoder::CanMsgAnalyseID_6E0(unsigned char * ucData, T_RBT_ACT_PSNG * tRbtActPsng)
{
    T_STATE_OUT_6E0 tStatout6E0 = { 0 };
//    unsigned char ucCanMsgSrc[8] = { 0 };

//    memcpy(ucCanMsgSrc, ucData, CAN_MSG_LEN);
    CharReverse(ucData,8);
    memcpy(&tStatout6E0, ucData, CAN_MSG_LEN);

    T_STATE_OUT_6E0_UNION tStateOut6E0Union;
    memset(&tStateOut6E0Union, 0, sizeof(T_STATE_OUT_6E0_UNION));
    tStateOut6E0Union.tStateOut6E0Src.iCcltStrActAngL = tStatout6E0.iCcltStrActAngL;
    tStateOut6E0Union.tStateOut6E0Src.iCcltStrActAngH = tStatout6E0.iCcltStrActAngH;

    tRbtActPsng->iCcltStrActAng = tStateOut6E0Union.tStateOut6E0Dest.iCcltStrActAng;
    tRbtActPsng->uiCcltAccActPsng = tStatout6E0.uiCcltAccActPsng;
    tRbtActPsng->uiCcltBrkActPsng = tStatout6E0.uiCcltBrkActPsng;
    tRbtActPsng->uiCcltClchPsng = tStatout6E0.uiCcltClchPsng;
}

//踏板学习位置反馈--数据解析
void MessageCoder::CanMsgAnalyseID_6E1(unsigned char * ucData, T_ACC_BRK_PSNG * tAccBrkPsng)
{
    T_STATE_OUT_6E1 tStatout6E1 = { 0 };
//    unsigned char ucCanMsgSrc[8] = { 0 };

//    memcpy(ucCanMsgSrc, ucData, CAN_MSG_LEN);
    CharReverse(ucData,8);
    memcpy(&tStatout6E1, ucData, CAN_MSG_LEN);

    tAccBrkPsng->uiClbtAccPsng0 = tStatout6E1.uiClbtAccPsng0;
    tAccBrkPsng->uiClbtAccPsngMax = tStatout6E1.uiClbtAccPsngMax;
    tAccBrkPsng->uiClbtBrkPsng0 = tStatout6E1.uiClbtBrkPsng0;
    tAccBrkPsng->uiClbtBrkPsngMax = tStatout6E1.uiClbtBrkPsngMax;
}

//转向学习位置反馈--数据解析
void MessageCoder::CanMsgAnalyseID_6E2(unsigned char * ucData, T_STR_LRN_PSNG * tStrLrnPsng)
{
    T_STATE_OUT_6E2 tStatout6E2 = { 0 };
//    unsigned char ucCanMsgSrc[8] = { 0 };

//    memcpy(ucCanMsgSrc, ucData, CAN_MSG_LEN);
    CharReverse(ucData,8);
    memcpy(&tStatout6E2, ucData, CAN_MSG_LEN);

    T_STATE_OUT_6E2_UNION tStateOut6E2Union;
    memset(&tStateOut6E2Union,0,sizeof(T_STATE_OUT_6E2_UNION));
    tStateOut6E2Union.tStateOut6E2Src.iClbtStrAngLftMaxL = tStatout6E2.iClbtStrAngLftMaxL;
    tStateOut6E2Union.tStateOut6E2Src.iClbtStrAngLftMaxH = tStatout6E2.iClbtStrAngLftMaxH;

    tStrLrnPsng->iClbtStrAng0 = tStatout6E2.iClbtStrAng0;
    tStrLrnPsng->iClbtStrAngLftMax = tStateOut6E2Union.tStateOut6E2Dest.iClbtStrAngLftMax;
    tStrLrnPsng->iClbtStrAngRtMax = tStatout6E2.iClbtStrAngRtMax;
}

//档位学习位置反馈1--数据解析
void MessageCoder::CanMsgAnalyseID_6E3(unsigned char * ucData, T_SFT_LRN_PST_1 * tSftLrnPst1)
{
    T_STATE_OUT_6E3 tStatout6E3 = { 0 };
//    unsigned char ucCanMsgSrc[8] = { 0 };

//    memcpy(ucCanMsgSrc, ucData, CAN_MSG_LEN);
    CharReverse(ucData,8);
    memcpy(&tStatout6E3, ucData, CAN_MSG_LEN);

    T_STATE_OUT_6E3_UNION tStateOut6E3Union;
    memset(&tStateOut6E3Union,0,sizeof(T_STATE_OUT_6E3_UNION));
    tStateOut6E3Union.tStateOut6E3Src.iRYMotPstL = tStatout6E3.iRYMotPstL;
    tStateOut6E3Union.tStateOut6E3Src.iRYMotPstH = tStatout6E3.iRYMotPstH;

    tSftLrnPst1->iDXMotPst = tStatout6E3.iDXMotPst;
    tSftLrnPst1->iNXMotPst = tStatout6E3.iNXMotPst;
    tSftLrnPst1->iNYMotPst = tStatout6E3.iNYMotPst;
    tSftLrnPst1->iPXMotPst = tStatout6E3.iPXMotPst;
    tSftLrnPst1->iPYMotPst = tStatout6E3.iPYMotPst;
    tSftLrnPst1->iRXMotPst = tStatout6E3.iRXMotPst;
    tSftLrnPst1->iRYMotPst = tStateOut6E3Union.tStateOut6E3Dest.iRYMotPst;
}

//控制器上电状态--数据解析
void MessageCoder::CanMsgAnalyseID_6E4(unsigned char * ucData, T_CTR_POW_STATUS * tCtrPowStatus)
{
    T_STATE_OUT_6E4 tStatout6E4 = { 0 };
//    unsigned char ucCanMsgSrc[8] = { 0 };

//    memcpy(ucCanMsgSrc, ucData, CAN_MSG_LEN);
    CharReverse(ucData,8);
    memcpy(&tStatout6E4, ucData, CAN_MSG_LEN);

    tCtrPowStatus->uiAccCtrPowStat = tStatout6E4.uiAccCtrPowStat;
    tCtrPowStatus->uiBrkCtrPowStat = tStatout6E4.uiBrkCtrPowStat;
    tCtrPowStatus->uiClchCtrPowStat = tStatout6E4.uiClchCtrPowStat;
    tCtrPowStatus->uiRCUPowStat = tStatout6E4.uiRCUPowStat;
    tCtrPowStatus->uiStrCtrPowStat = tStatout6E4.uiStrCtrPowStat;
    tCtrPowStatus->uiXSftArmCtrPowStat = tStatout6E4.uiXSftArmCtrPowStat;
    tCtrPowStatus->uiYSftArmCtrPowStat = tStatout6E4.uiYSftArmCtrPowStat;
    tCtrPowStatus->uiZSftArmCtrPowStat = tStatout6E4.uiZSftArmCtrPowStat;
}

//执行机构使能状态--数据解析
void MessageCoder::CanMsgAnalyseID_6E5(unsigned char * ucData, T_ACT_ENABLE_STATUS * tActEnableStatus)
{
    T_STATE_OUT_6E5 tStatout6E5 = { 0 };
//    unsigned char ucCanMsgSrc[8] = { 0 };

//    memcpy(ucCanMsgSrc, ucData, CAN_MSG_LEN);
    CharReverse(ucData,8);
    memcpy(&tStatout6E5, ucData, CAN_MSG_LEN);

    tActEnableStatus->uiAccClchCtrStat = tStatout6E5.uiAccClchCtrStat;
    tActEnableStatus->uiAccMotEnStat = tStatout6E5.uiAccMotEnStat;
    tActEnableStatus->uiBrkMotEnStat = tStatout6E5.uiBrkMotEnStat;
    tActEnableStatus->uiClchMotEnStat = tStatout6E5.uiClchMotEnStat;
    tActEnableStatus->uiPauseEnStat = tStatout6E5.uiPauseEnStat;
    tActEnableStatus->uiStrMotEnStat = tStatout6E5.uiStrMotEnStat;
    tActEnableStatus->uiXSftMotEnStat = tStatout6E5.uiXSftMotEnStat;
    tActEnableStatus->uiYSftMotEnStat = tStatout6E5.uiYSftMotEnStat;
    tActEnableStatus->uiZSftMotEnStat = tStatout6E5.uiZSftMotEnStat;
}

//油门控制状态反馈--数据解析
void MessageCoder::CanMsgAnalyseID_6E6(unsigned char * ucData, T_ACC_CTR_STATUS * tAccCtrStatus)
{
    T_STATE_OUT_6E6 tStatout6E6 = { 0 };
//    unsigned char ucCanMsgSrc[8] = { 0 };

//    memcpy(ucCanMsgSrc, ucData, CAN_MSG_LEN);
    CharReverse(ucData,8);
    memcpy(&tStatout6E6, ucData, CAN_MSG_LEN);

    tAccCtrStatus->iAccDspSpd = tStatout6E6.iAccDspSpd;
    tAccCtrStatus->iAccMotSpd = tStatout6E6.iAccMotSpd;
    tAccCtrStatus->iAccMotTrq = tStatout6E6.iAccMotTrq;
}

//制动踏板控制状态反馈--数据解析
void MessageCoder::CanMsgAnalyseID_6E7(unsigned char * ucData, T_BRK_CTR_STATUS * tBrkCtrStatus)
{
    T_STATE_OUT_6E7 tStatout6E7 = { 0 };
//    unsigned char ucCanMsgSrc[8] = { 0 };

//    memcpy(ucCanMsgSrc, ucData, CAN_MSG_LEN);
    CharReverse(ucData,8);
    memcpy(&tStatout6E7, ucData, CAN_MSG_LEN);

    tBrkCtrStatus->iBrkDspSpd = tStatout6E7.iBrkDspSpd;
    tBrkCtrStatus->iBrkFoc = tStatout6E7.iBrkFoc;
    tBrkCtrStatus->iBrkMotSpd = tStatout6E7.iBrkMotSpd;
    tBrkCtrStatus->iBrkMotTrq = tStatout6E7.iBrkMotTrq;
    tBrkCtrStatus->uiBrkCtrMd = tStatout6E7.uiBrkCtrMd;

}

//离合踏板控制状态反馈--数据解析
void MessageCoder::CanMsgAnalyseID_6E8(unsigned char * ucData, T_CLCH_CTR_STATUS * tClchCtrStatus)
{
    T_STATE_OUT_6E8 tStatout6E8 = { 0 };
//    unsigned char ucCanMsgSrc[8] = { 0 };

//    memcpy(ucCanMsgSrc, ucData, CAN_MSG_LEN);
    CharReverse(ucData,8);
    memcpy(&tStatout6E8, ucData, CAN_MSG_LEN);

    T_STATE_OUT_6E8_UNION tStateOut6E8Union;
    memset(&tStateOut6E8Union,0,sizeof(T_STATE_OUT_6E8_UNION));
    tStateOut6E8Union.tStateOut6E8Src.uiClchFocH = tStatout6E8.uiClchFocH;
    tStateOut6E8Union.tStateOut6E8Src.uiClchFocL = tStatout6E8.uiClchFocL;

    tClchCtrStatus->iClchDspSpd = tStatout6E8.iClchDspSpd;
    tClchCtrStatus->iClchMotSpd = tStatout6E8.iClchMotSpd;
    tClchCtrStatus->iClchMotTrq = tStatout6E8.iClchMotTrq;
    tClchCtrStatus->uiClchFoc = tStateOut6E8Union.tStateOut6E8Dest.uiClchFoc;
}

//转向控制状态反馈--数据解析
void MessageCoder::CanMsgAnalyseID_6E9(unsigned char * ucData, T_STR_STATUS * tStrStatus)
{
    T_STATE_OUT_6E9 tStatout6E9 = { 0 };
//    unsigned char ucCanMsgSrc[8] = { 0 };

//    memcpy(ucCanMsgSrc, ucData, CAN_MSG_LEN);
    CharReverse(ucData,8);
    memcpy(&tStatout6E9, ucData, CAN_MSG_LEN);

    T_STATE_OUT_6E9_UNION tStateOut6E9Union;
    memset(&tStateOut6E9Union,0,sizeof(T_STATE_OUT_6E9_UNION));
    tStateOut6E9Union.tStateOut6E9Src.iStrMotSpdH = tStatout6E9.iStrMotSpdH;
    tStateOut6E9Union.tStateOut6E9Src.iStrMotSpdL = tStatout6E9.iStrMotSpdL;

    tStrStatus->iCcltStrSpd = tStatout6E9.iCcltStrSpd;
    tStrStatus->iStrMotSpd = tStateOut6E9Union.tStateOut6E9Dest.iStrMotSpd;
    tStrStatus->iStrMotTrq = tStatout6E9.iStrMotTrq;
    tStrStatus->iStrTrq = tStatout6E9.iStrTrq;
    tStrStatus->uiStrCtrMd = tStatout6E9.uiStrCtrMd;
}

//档位控制状态反馈--数据解析
void MessageCoder::CanMsgAnalyseID_6EA(unsigned char * ucData, T_SFT_STATUS * tSftStatus)
{
    T_STATE_OUT_6EA tStatout6EA = { 0 };
//    unsigned char ucCanMsgSrc[8] = { 0 };

//    memcpy(ucCanMsgSrc, ucData, CAN_MSG_LEN);
    CharReverse(ucData,8);
    memcpy(&tStatout6EA, ucData, CAN_MSG_LEN);

    T_STATE_OUT_6EA_UNION tStateOut6EAUnion;
    memset(&tStateOut6EAUnion, 0, sizeof(T_STATE_OUT_6EA_UNION));

    tStateOut6EAUnion.tStateOut6EASrc.iXSftMotSpdH = tStatout6EA.iXSftMotSpdH;
    tStateOut6EAUnion.tStateOut6EASrc.iXSftMotSpdL = tStatout6EA.iXSftMotSpdL;

    tSftStatus->iXSftMotAng = tStatout6EA.iXSftMotAng;
    tSftStatus->iXSftMotSpd = tStateOut6EAUnion.tStateOut6EADest.iXSftMotSpd;
    tSftStatus->iXSftMotTrq = tStatout6EA.iXSftMotTrq;
    tSftStatus->iYSftMotAng = tStatout6EA.iYSftMotAng;
    tSftStatus->iYSftMotSpd = tStatout6EA.iYSftMotSpd;
    tSftStatus->iYSftMotTrq = tStatout6EA.iYSftMotTrq;
    tSftStatus->uiRbtSftPsng = tStatout6EA.uiRbtSftPsng;
    tSftStatus->uiSftSpdType = tStatout6EA.uiSftSpdType;
}

//系统运行状态参数反馈1--数据解析
void MessageCoder::CanMsgAnalyseID_6EB(unsigned char * ucData, T_SYS_INFO_1 * tSysInfo1)
{
    T_STATE_OUT_6EB tStatout6EB = { 0 };
//    unsigned char ucCanMsgSrc[8] = { 0 };

//    memcpy(ucCanMsgSrc, ucData, CAN_MSG_LEN);
    CharReverse(ucData,8);
    memcpy(&tStatout6EB, ucData, CAN_MSG_LEN);

    tSysInfo1->uiAccMotCur = tStatout6EB.uiAccMotCur;
    tSysInfo1->uiBrkMotCur = tStatout6EB.uiBrkMotCur;
    tSysInfo1->uiClchMotCur = tStatout6EB.uiClchMotCur;
    tSysInfo1->uiRbtRunTime = tStatout6EB.uiRbtRunTime;
    tSysInfo1->uiStrMotCur = tStatout6EB.uiStrMotCur;
    tSysInfo1->uiXSftMotCur = tStatout6EB.uiXSftMotCur;
    tSysInfo1->uiYSftMotCur = tStatout6EB.uiYSftMotCur;
}

//系统运行状态参数反馈2--数据解析
void MessageCoder::CanMsgAnalyseID_6ED(unsigned char * ucData, T_SYS_INFO_2 * tSysInfo2)
{
    T_STATE_OUT_6ED tStatout6ED = { 0 };
//    unsigned char ucCanMsgSrc[8] = { 0 };

//    memcpy(ucCanMsgSrc, ucData, CAN_MSG_LEN);
    CharReverse(ucData,8);
    memcpy(&tStatout6ED, ucData, CAN_MSG_LEN);

    tSysInfo2->uiAccCtrAlive = tStatout6ED.uiAccCtrAlive;
    tSysInfo2->uiAccMotClbr = tStatout6ED.uiAccMotClbr;
    tSysInfo2->uiBrkCtrAlive = tStatout6ED.uiBrkCtrAlive;
    tSysInfo2->uiBrkMotClbr = tStatout6ED.uiBrkMotClbr;
    tSysInfo2->uiClchCtrAlive = tStatout6ED.uiClchCtrAlive;
    tSysInfo2->uiClchMotClbr = tStatout6ED.uiClchMotClbr;
    tSysInfo2->uiCtrMdStat = tStatout6ED.uiCtrMdStat;
    tSysInfo2->uiRCUAlive = tStatout6ED.uiRCUAlive;
    tSysInfo2->uiStrCtrAlive = tStatout6ED.uiStrCtrAlive;
    tSysInfo2->uiStrMotClbr = tStatout6ED.uiStrMotClbr;
    tSysInfo2->uiXSftArmCtrAlive = tStatout6ED.uiXSftArmCtrAlive;
    tSysInfo2->uiXSftMotClbr = tStatout6ED.uiXSftMotClbr;
    tSysInfo2->uiYSftArmCtrAlive = tStatout6ED.uiYSftArmCtrAlive;
    tSysInfo2->uiYSftMotClbr = tStatout6ED.uiYSftMotClbr;
    tSysInfo2->uiRbtCfgMdStat = tStatout6ED.uiRbtCfgMdStat;

}

//系统学习状态反馈--数据解析
void MessageCoder::CanMsgAnalyseID_6EE(unsigned char * ucData, T_SYS_LRN * tSysLrn)
{
    T_STATE_OUT_6EE tStatout6EE = { 0 };
//    unsigned char ucCanMsgSrc[8] = { 0 };

//    memcpy(ucCanMsgSrc, ucData, CAN_MSG_LEN);
    CharReverse(ucData,8);
    memcpy(&tStatout6EE, ucData, CAN_MSG_LEN);

    tSysLrn->uiAcc0LrnStat = tStatout6EE.uiAcc0LrnStat;
    tSysLrn->uiAccLrnStat = tStatout6EE.uiAccLrnStat;
    tSysLrn->uiAccMaxLrnStat = tStatout6EE.uiAccMaxLrnStat;
    tSysLrn->uiBrk0LrnStat = tStatout6EE.uiBrk0LrnStat;
    tSysLrn->uiBrkLrnStat = tStatout6EE.uiBrkLrnStat;
    tSysLrn->uiBrkMaxLrnStat = tStatout6EE.uiBrkMaxLrnStat;
    tSysLrn->uiClch0LrnStat = tStatout6EE.uiClch0LrnStat;
    tSysLrn->uiClchLrnStat = tStatout6EE.uiClchLrnStat;
    tSysLrn->uiClchMaxLrnStat = tStatout6EE.uiClchMaxLrnStat;
    tSysLrn->uiDLrnStat = tStatout6EE.uiDLrnStat;
    tSysLrn->uiDpLrnStat = tStatout6EE.uiDpLrnStat;
    tSysLrn->uiMLrnStat = tStatout6EE.uiMLrnStat;
    tSysLrn->uiNLrnStat = tStatout6EE.uiNLrnStat;
    tSysLrn->uiStrLrnStat = tStatout6EE.uiStrLrnStat;
    tSysLrn->uiSftLrnStat = tStatout6EE.uiSftLrnStat;
    tSysLrn->uiPLrnStat = tStatout6EE.uiPLrnStat;
    tSysLrn->uiRLrnStat = tStatout6EE.uiRLrnStat;
    tSysLrn->uiUpLrnStat= tStatout6EE.uiUpLrnStat;
    tSysLrn->uiStr0LrnStat = tStatout6EE.uiStr0LrnStat;
    tSysLrn->uiStrLLrnStat = tStatout6EE.uiStrLLrnStat;
    tSysLrn->uiStrRLrnStat = tStatout6EE.uiStrRLrnStat;
    tSysLrn->uiAccLrnWay = tStatout6EE.uiAccLrnWay;
    tSysLrn->uiBrkLrnWay = tStatout6EE.uiBrkLrnWay;
    tSysLrn->uiStrLrnWay = tStatout6EE.uiStrLrnWay;
    tSysLrn->uiClutchLrnWay = tStatout6EE.uiClutchLrnWay;
}

//档位学习位置反馈2--数据解析
void MessageCoder::CanMsgAnalyseID_6EF(unsigned char * ucData, T_SFT_LRN_PST_2 * tSFtLrnPst2)
{
    T_STATE_OUT_6EF tStatout6EF = { 0 };
//    unsigned char ucCanMsgSrc[8] = { 0 };

//    memcpy(ucCanMsgSrc, ucData, CAN_MSG_LEN);
    CharReverse(ucData,8);
    memcpy(&tStatout6EF, ucData, CAN_MSG_LEN);

    T_STATE_OUT_6EF_UNION tStateOut6EFUnion;
    memset(&tStateOut6EFUnion, 0, sizeof(T_STATE_OUT_6EF_UNION));
    tStateOut6EFUnion.tStateOut6EFSrc.iUpXMotPstH = tStatout6EF.iUpXMotPstH;
    tStateOut6EFUnion.tStateOut6EFSrc.iUpXMotPstL = tStatout6EF.iUpXMotPstL;

    tSFtLrnPst2->iDownXMotPst = tStatout6EF.iDownXMotPst;
    tSFtLrnPst2->iDownYMotPst = tStatout6EF.iDownYMotPst;
    tSFtLrnPst2->iDYMotPst = tStatout6EF.iDYMotPst;
    tSFtLrnPst2->iMXMotPst = tStatout6EF.iMXMotPst;
    tSFtLrnPst2->iMYMotPst = tStatout6EF.iMYMotPst;
    tSFtLrnPst2->iUpXMotPst = tStateOut6EFUnion.tStateOut6EFDest.iUpXMotPst;
    tSFtLrnPst2->iUpYMotPst = tStatout6EF.iMYMotPst;
}

//系统监控状态反馈--数据解析
void MessageCoder::CanMsgAnalyseID_6D0(unsigned char * ucData, T_SYS_MONITOR * tSysMonitor)
{
    T_STATE_OUT_6D0 tStatout6D0 = { 0 };
//    unsigned char ucCanMsgSrc[8] = { 0 };

//    memcpy(ucCanMsgSrc, ucData, CAN_MSG_LEN);
    CharReverse(ucData,8);
    memcpy(&tStatout6D0, ucData, CAN_MSG_LEN);

    tSysMonitor->iCtrTmp = tStatout6D0.iCtrTmp;
    tSysMonitor->uiRbtSlfChkStat = tStatout6D0.uiRbtSlfChkStat;
    tSysMonitor->uiRCUHdwAbnStat = tStatout6D0.uiRCUHdwAbnStat;
    tSysMonitor->uiRCUSftwAbnStat = tStatout6D0.uiRCUSftwAbnStat;
    tSysMonitor->uiSysAbnStat = tStatout6D0.uiSysAbnStat;
    tSysMonitor->uiAccAbnStat = tStatout6D0.uiAccAbnStat;
    tSysMonitor->uiBrkAbnStat = tStatout6D0.uiBrkAbnStat;
    tSysMonitor->uiStrAbnStat = tStatout6D0.uiStrAbnStat;
    tSysMonitor->uiClchAbnStat = tStatout6D0.uiClchAbnStat;
    tSysMonitor->uiXSftAbnStat = tStatout6D0.uiXSftAbnStat;
    tSysMonitor->uiYSftAbnStat = tStatout6D0.uiYSftAbnStat;
    tSysMonitor->uiDrvCtrCanState = tStatout6D0.uiDrvCtrCanState;
    tSysMonitor->uiIOCtrCommunicateState = tStatout6D0.uiIOCtrCommunicateState;
}

//系统报警--数据解析
void MessageCoder::CanMsgAnalyseID_6D1(unsigned char * ucData, T_SYS_ERR * tSysErr)
{
    T_STATE_OUT_6D1 tStatOut6D1 = { 0 };
//    unsigned char ucCanMsgSrc[8] = { 0 };

//    memcpy(ucCanMsgSrc, ucData, CAN_MSG_LEN);
    CharReverse(ucData,8);
    memcpy(&tStatOut6D1, ucData, CAN_MSG_LEN);

    tSysErr->uiSysErra = tStatOut6D1.uiSysErra;
    tSysErr->uiSysErrb = tStatOut6D1.uiSysErrb;
    tSysErr->uiSysErrc = tStatOut6D1.uiSysErrc;
    tSysErr->uiSysErrd = tStatOut6D1.uiSysErrd;
    tSysErr->uiSysErre = tStatOut6D1.uiSysErre;
    tSysErr->uiSysErrf = tStatOut6D1.uiSysErrf;
    tSysErr->uiSysErrg = tStatOut6D1.uiSysErrg;
    tSysErr->uiSysErrh = tStatOut6D1.uiSysErrh;
    tSysErr->uiSysErri = tStatOut6D1.uiSysErri;
    tSysErr->uiSysErrj = tStatOut6D1.uiSysErrj;
    tSysErr->uiSysErrk = tStatOut6D1.uiSysErrk;
    tSysErr->uiSysErrl = tStatOut6D1.uiSysErrl;
    tSysErr->uiSysErrm = tStatOut6D1.uiSysErrm;
    tSysErr->uiSysErrn = tStatOut6D1.uiSysErrn;
    tSysErr->uiSysErro = tStatOut6D1.uiSysErro;
    tSysErr->uiSysErrp = tStatOut6D1.uiSysErrp;
    tSysErr->uiSysErrq = tStatOut6D1.uiSysErrq;
    tSysErr->uiSysErrr = tStatOut6D1.uiSysErrr;
    tSysErr->uiSysErrs = tStatOut6D1.uiSysErrs;
    tSysErr->uiSysErrt = tStatOut6D1.uiSysErrt;
}

//方向盘位置时间控制请求--数据解析
void MessageCoder::CanMsgAnalyseID_6D7(unsigned char * ucData, T_STR_PST_TIME_REQ * tSteerPstTimeReq)
{
    T_STATE_OUT_6D7 tCtrCmd6D7 = { 0 };
//    unsigned char ucCanMsgSrc[8] = { 0 };

//    memcpy(ucCanMsgSrc, ucData, CAN_MSG_LEN);
    CharReverse(ucData,8);
    memcpy(&tCtrCmd6D7, ucData, CAN_MSG_LEN);

    tSteerPstTimeReq->uiNumReq = tCtrCmd6D7.uiNumReq;
    tSteerPstTimeReq->uiSequenceIDReq = tCtrCmd6D7.uiSequenceIDReq;
}

//Estop系统信息--数据解析
void MessageCoder::CanMsgAnalyseID_2E0(unsigned char * ucData, T_ESTOP_INFO * tEstopInfo)
{
    T_ESTOP_2E0 tEstop2E0 = { 0 };
//    unsigned char ucCanMsgSrc[8] = { 0 };

//    memcpy(ucCanMsgSrc, ucData, CAN_MSG_LEN);
    CharReverse(ucData,8);
    memcpy(&tEstop2E0, ucData, CAN_MSG_LEN);

    tEstopInfo->uiEstpAbnStat = tEstop2E0.uiEstpAbnStat;
    tEstopInfo->uiEstpAlive = tEstop2E0.uiEstpAlive;
    tEstopInfo->uiEstpEnStat = tEstop2E0.uiEstpEnStat;
    tEstopInfo->uiPmpEnStat = tEstop2E0.uiPmpEnStat;
    tEstopInfo->uiPmpPressureStat = tEstop2E0.uiPmpPressureStat;
    tEstopInfo->uiPmpValveStat = tEstop2E0.uiPmpValveStat;
}

//Estop故障码--数据解析
void MessageCoder::CanMsgAnalyseID_2E1(unsigned char * ucData, T_ESTOP_ERR * tEstopErr)
{
    T_ESTOP_2E1 tEstop2E1 = { 0 };
//    unsigned char ucCanMsgSrc[8] = { 0 };

//    memcpy(ucCanMsgSrc, ucData, CAN_MSG_LEN);
    CharReverse(ucData,8);
    memcpy(&tEstop2E1, ucData, CAN_MSG_LEN);

    tEstopErr->uiEspErra = tEstop2E1.uiEspErra;
    tEstopErr->uiEspErrb = tEstop2E1.uiEspErrb;
    tEstopErr->uiEspErrc = tEstop2E1.uiEspErrc;
    tEstopErr->uiEspErrd = tEstop2E1.uiEspErrd;
    tEstopErr->uiEspErre = tEstop2E1.uiEspErre;
    tEstopErr->uiEspErrf = tEstop2E1.uiEspErrf;
}

//油门(acc)、制动(brk)、挡位(sft)X、挡位(sft)Y、转向(str)电机报警--数据解析
void MessageCoder::CanMsgAnalyseErr(unsigned char * ucData, T_MOT_CTR_ERR * tMotCtrErr)
{
    T_CTR_ERR_CODE tCtrErrCode = { 0 };
//    unsigned char ucCanMsgSrc[8] = { 0 };

//    memcpy(ucCanMsgSrc, ucData, CAN_MSG_LEN);
    CharReverse(ucData,8);
    memcpy(&tCtrErrCode, ucData, CAN_MSG_LEN);

    tMotCtrErr->uiErra = tCtrErrCode.a;
    tMotCtrErr->uiErraa = tCtrErrCode.aa;
    tMotCtrErr->uiErrab = tCtrErrCode.ab;
    tMotCtrErr->uiErrac = tCtrErrCode.ac;
    tMotCtrErr->uiErrad = tCtrErrCode.ad;
    tMotCtrErr->uiErrae = tCtrErrCode.ae;
    tMotCtrErr->uiErraf = tCtrErrCode.af;
    tMotCtrErr->uiErrag = tCtrErrCode.ag;
    tMotCtrErr->uiErrah = tCtrErrCode.ah;
    tMotCtrErr->uiErrai = tCtrErrCode.ai;
    tMotCtrErr->uiErrb = tCtrErrCode.b;
    tMotCtrErr->uiErrc = tCtrErrCode.c;
    tMotCtrErr->uiErrd = tCtrErrCode.d;
    tMotCtrErr->uiErre = tCtrErrCode.e;
    tMotCtrErr->uiErrf = tCtrErrCode.f;
    tMotCtrErr->uiErrg = tCtrErrCode.g;
    tMotCtrErr->uiErrh = tCtrErrCode.h;
    tMotCtrErr->uiErri = tCtrErrCode.i;
    tMotCtrErr->uiErrj = tCtrErrCode.j;
    tMotCtrErr->uiErrk = tCtrErrCode.k;
    tMotCtrErr->uiErrl = tCtrErrCode.l;
    tMotCtrErr->uiErrm = tCtrErrCode.m;
    tMotCtrErr->uiErrn = tCtrErrCode.n;
    tMotCtrErr->uiErro = tCtrErrCode.o;
    tMotCtrErr->uiErrp = tCtrErrCode.p;
    tMotCtrErr->uiErrq = tCtrErrCode.q;
    tMotCtrErr->uiErrr = tCtrErrCode.r;
    tMotCtrErr->uiErrs = tCtrErrCode.s;
    tMotCtrErr->uiErrt = tCtrErrCode.t;
    tMotCtrErr->uiErru = tCtrErrCode.u;
    tMotCtrErr->uiErrv = tCtrErrCode.v;
    tMotCtrErr->uiErrw = tCtrErrCode.w;
    tMotCtrErr->uiErrx = tCtrErrCode.x;
    tMotCtrErr->uiErry = tCtrErrCode.y;
    tMotCtrErr->uiErrz = tCtrErrCode.z;
}

//OBD实际油门踏板行程--数据解析
void MessageCoder::OBDAnalyseID_070(unsigned char *ucData, T_OBD_ACC_PSNG *tOBDAccPsng)
{
    T_OBD_070 tOBD070 = { 0 };
    unsigned char ucCanMsgSrc[8] = { 0 };

    memcpy(ucCanMsgSrc, ucData, 8);
    memcpy(&tOBD070, ucCanMsgSrc, 8);
    unsigned int psng = tOBD070.obdAccPsng -5;
    if(psng > 2500)
    {
      tOBDAccPsng->obdAccPsng = 100;
    }
    else
    {
      tOBDAccPsng->obdAccPsng = psng / 2500 * 1000;
    }
}


//控制器上电命令--数据组包
void MessageCoder::CanMsgPackageID_7F2(T_POWER_ON_REQUEST * tPowOnReq, TPCANMsg *tCanMsg)
{
    T_CONTROL_CMD_7F2 tCtrCmd7F2 = { 0 };
    unsigned char ucCanMsgSrc[8] = { 0 };

    tCtrCmd7F2.uiAccCtrPowReq = tPowOnReq->uiAccCtrPowReq;
    tCtrCmd7F2.uiBrkCtrPowReq = tPowOnReq->uiBrkCtrPowReq;
    tCtrCmd7F2.uiClchCtrPowReq = tPowOnReq->uiClchCtrPowReq;
    tCtrCmd7F2.uiRCUPowReq = tPowOnReq->uiRCUPowReq;
    tCtrCmd7F2.uiStrCtrPowReq = tPowOnReq->uiStrCtrPowReq;
    tCtrCmd7F2.uiXSftCtrPowReq = tPowOnReq->uiXSftCtrPowReq;
    tCtrCmd7F2.uiYSftCtrPowReq = tPowOnReq->uiYSftCtrPowReq;
    tCtrCmd7F2.uiDriveFanReq = tPowOnReq->uiDriveFanReq;
    tCtrCmd7F2.uiZSftArmCtrPowReq = tPowOnReq->uiZSftArmCtrPowReq;

    memcpy(ucCanMsgSrc,&tCtrCmd7F2,CAN_MSG_LEN);
        CharReverse(ucCanMsgSrc,8);
    memcpy(tCanMsg->DATA, ucCanMsgSrc, CAN_MSG_LEN);
    tCanMsg->ID = robot::common::PowerOnReq;
    tCanMsg->LEN = 8;
    tCanMsg->MSGTYPE = MSGTYPE_STANDARD;
}

//执行机构使能请求--数据组包
void MessageCoder::CanMsgPackageID_7F5(T_ACTUATOR_ENABLE_REQUEST *tActEnableReq, TPCANMsg *tCanMsg)
{
    T_CONTROL_CMD_7F5 tCtrCmd7F5 = { 0 };
    unsigned char ucCanMsgSrc[8] = { 0 };

    tCtrCmd7F5.uiAccClchCtrReq = tActEnableReq->uiAccClchCtrReq;
    tCtrCmd7F5.uiAccMotEnReq = tActEnableReq->uiAccMotEnReq;
    tCtrCmd7F5.uiBrkMotEnReq = tActEnableReq->uiBrkMotEnReq;
    tCtrCmd7F5.uiClchMotEnReq = tActEnableReq->uiClchMotEnReq;
    tCtrCmd7F5.uiStrMotEnReq = tActEnableReq->uiStrMotEnReq;
    tCtrCmd7F5.uiXSftMotEnReq = tActEnableReq->uiXSftMotEnReq;
    tCtrCmd7F5.uiYSftMotEnReq = tActEnableReq->uiYSftMotEnReq;
    tCtrCmd7F5.uiZSftMotEnReq = tActEnableReq->uiZSftMotEnReq;

    memcpy(ucCanMsgSrc, &tCtrCmd7F5, CAN_MSG_LEN);
    CharReverse(ucCanMsgSrc,8);
    memcpy(tCanMsg->DATA, ucCanMsgSrc, CAN_MSG_LEN);
    tCanMsg->ID = robot::common::ActuatorEnableReq;
    tCanMsg->LEN = 8;
    tCanMsg->MSGTYPE = MSGTYPE_STANDARD;
}

//学习模式请求确认--数据组包
void MessageCoder::CanMsgPackageID_7F7(T_LEARN_MODE_REQUEST * tLearnMdReq, TPCANMsg *tCanMsg)
{
    T_CONTROL_CMD_7F7 tCtrCmd7F7 = { 0 };
    unsigned char ucCanMsgSrc[8] = { 0 };

    tCtrCmd7F7.uiClrAccTbl = tLearnMdReq->uiClrAccTbl;
    tCtrCmd7F7.uiClrBrkTbl = tLearnMdReq->uiClrBrkTbl;
    tCtrCmd7F7.uiClrClchTbl = tLearnMdReq->uiClrClchTbl;
    tCtrCmd7F7.uiClrSftTbl = tLearnMdReq->uiClrSftTbl;
    tCtrCmd7F7.uiClrStrTbl = tLearnMdReq->uiClrStrTbl;
    tCtrCmd7F7.uiPsngLrnReq = tLearnMdReq->uiPsngLrnReq;
    tCtrCmd7F7.uiSftAutLrnFnsConf = tLearnMdReq->uiSftAutLrnFnsConf;
    tCtrCmd7F7.uiSftPsngLrnReq = tLearnMdReq->uiSftPsngLrnReq;
    tCtrCmd7F7.uiStpAutLrn = tLearnMdReq->uiStpAutLrn;
    tCtrCmd7F7.uiSysLrnReq = tLearnMdReq->uiSysLrnReq;

    memcpy(ucCanMsgSrc, &tCtrCmd7F7, CAN_MSG_LEN);
    CharReverse(ucCanMsgSrc,8);
    memcpy(tCanMsg->DATA, ucCanMsgSrc, CAN_MSG_LEN);
    tCanMsg->ID = robot::common::LearnModeReq;
    tCanMsg->LEN = 8;
    tCanMsg->MSGTYPE = MSGTYPE_STANDARD;
}

//油门踏板执行机构控制命令--数据组包
void MessageCoder::CanMsgPackageID_5F2(T_ACC_PEDAL_ACT_CTR_REQ * tAccPedalActCtrReq, TPCANMsg *tCanMsg)
{
    T_CONTROL_CMD_5F2 tCtrCmd5F2 = { 0 };
    unsigned char ucCanMsgSrc[8] = { 0 };

    tCtrCmd5F2.uiAccAccCtrReq = tAccPedalActCtrReq->uiAccAccCtrReq;
    tCtrCmd5F2.uiAccDecCtrReq = tAccPedalActCtrReq->uiAccDecCtrReq;
    tCtrCmd5F2.uiAccPsngCtrReq = tAccPedalActCtrReq->uiAccPsngCtrReq;
    tCtrCmd5F2.uiAccSpdCtrReq = tAccPedalActCtrReq->uiAccSpdCtrReq;

    memcpy(ucCanMsgSrc, &tCtrCmd5F2, CAN_MSG_LEN);
        CharReverse(ucCanMsgSrc,8);
    memcpy(tCanMsg->DATA, ucCanMsgSrc, CAN_MSG_LEN);
    tCanMsg->ID = robot::common::AccPedalActCtrReq;
    tCanMsg->LEN = 8;
    tCanMsg->MSGTYPE = MSGTYPE_STANDARD;
}

//制动踏板执行机构控制命令--数据组包
void MessageCoder::CanMsgPackageID_5F0(T_BRK_PEDAL_ACT_CTR_REQ * tBrkPedalActCtrReq, TPCANMsg *tCanMsg)
{
    T_CONTROL_CMD_5F0 tCtrCmd5F0 = { 0 };
    unsigned char ucCanMsgSrc[8] = { 0 };

    T_CONTROL_CMD_5F0_UNION tCtrCmd5F0Union;
    memset(&tCtrCmd5F0Union, 0, sizeof(T_CONTROL_CMD_5F0_UNION));
    tCtrCmd5F0Union.tCtrCmd5F0Dest.uiBrkFocCtrReq = tBrkPedalActCtrReq->uiBrkFocCtrReq;

    tCtrCmd5F0.uiBrkSpdCtrReq = tBrkPedalActCtrReq->uiBrkSpdCtrReq;
    tCtrCmd5F0.uiBrkAccCtrReq = tBrkPedalActCtrReq->uiBrkAccCtrReq;
    tCtrCmd5F0.uiBrkCtrMdReq = tBrkPedalActCtrReq->uiBrkCtrMdReq;
    tCtrCmd5F0.uiBrkDecCtrReq = tBrkPedalActCtrReq->uiBrkDecCtrReq;
    tCtrCmd5F0.uiBrkPsngCtrReq = tBrkPedalActCtrReq->uiBrkPsngCtrReq;
    tCtrCmd5F0.uiBrkFocCtrReqH = tCtrCmd5F0Union.tCtrCmd5F0Src.uiBrkFocCtrReqH;
    tCtrCmd5F0.uiBrkFocCtrReqL = tCtrCmd5F0Union.tCtrCmd5F0Src.uiBrkFocCtrReqL;

    memcpy(ucCanMsgSrc, &tCtrCmd5F0, CAN_MSG_LEN);
        CharReverse(ucCanMsgSrc,8);
    memcpy(tCanMsg->DATA, ucCanMsgSrc, CAN_MSG_LEN);
    tCanMsg->ID = robot::common::BrkPedalActCtrReq;
    tCanMsg->LEN = 8;
    tCanMsg->MSGTYPE = MSGTYPE_STANDARD;
}

//转向执行机构控制命令--数据组包
void MessageCoder::CanMsgPackageID_5F4(T_STR_ACT_CTR_REQ * tStrActCtrReq, TPCANMsg *tCanMsg)
{
    T_CONTROL_CMD_5F4 tCtrCmd5F4 = { 0 };
    unsigned char ucCanMsgSrc[8] = { 0 };

    T_CONTROL_CMD_5F4_UNION tCtrCmd5F4Union;
    memset(&tCtrCmd5F4Union, 0, sizeof(T_CONTROL_CMD_5F4_UNION));
    tCtrCmd5F4Union.tCtrCmd5F4Dest.iStrSpdCtrReq = tStrActCtrReq->iStrSpdCtrReq;

    tCtrCmd5F4.iStrAngCtrReq = tStrActCtrReq->iStrAngCtrReq;
    tCtrCmd5F4.iStrSpdCtrReqH = tCtrCmd5F4Union.tCtrCmd5F4Src.iStrSpdCtrReqH;
    tCtrCmd5F4.iStrSpdCtrReqL = tCtrCmd5F4Union.tCtrCmd5F4Src.iStrSpdCtrReqL;
    tCtrCmd5F4.iStrTrqCtrReq = tStrActCtrReq->iStrTrqCtrReq;
    tCtrCmd5F4.uiStrCtrMdReq = tStrActCtrReq->uiStrCtrMdReq;
    memcpy(ucCanMsgSrc, &tCtrCmd5F4, CAN_MSG_LEN);
    CharReverse(ucCanMsgSrc,8);
    memcpy(tCanMsg->DATA, ucCanMsgSrc, CAN_MSG_LEN);
    tCanMsg->ID = robot::common::SteerActCtrReq;
    tCanMsg->LEN = 8;
    tCanMsg->MSGTYPE = MSGTYPE_STANDARD;
}

//档位执行机构控制命令--数据组包
void MessageCoder::CanMsgPackageID_5FA(T_SFT_ACT_CTR_REQ * tSftActCtrReq, TPCANMsg *tCanMsg)
{
    T_CONTROL_CMD_5FA tCtrCmd5FA = { 0 };
    unsigned char ucCanMsgSrc[8] = { 0 };

    tCtrCmd5FA.iClchSpdCtrReq = tSftActCtrReq->iClchSpdCtrReq;
    tCtrCmd5FA.uiClchPsngCtrReq = tSftActCtrReq->uiClchPsngCtrReq;
    tCtrCmd5FA.uiSftMdReq = tSftActCtrReq->uiSftMdReq;
    tCtrCmd5FA.uiSftPsngCtrReq = tSftActCtrReq->uiSftPsngCtrReq;

    memcpy(ucCanMsgSrc, &tCtrCmd5FA, CAN_MSG_LEN);
    CharReverse(ucCanMsgSrc,8);
    memcpy(tCanMsg->DATA, ucCanMsgSrc, CAN_MSG_LEN);
    tCanMsg->ID = robot::common::ShiftActCtrReq;
    tCanMsg->LEN = 8;
    tCanMsg->MSGTYPE = MSGTYPE_STANDARD;
}

//机器人自检控制命令--数据组包
void MessageCoder::CanMsgPackageID_5FC(T_RBT_SLF_CHK_CTR_REQ * tRbtSlfChkCtrRq, TPCANMsg *tCanMsg)
{
    T_CONTROL_CMD_5FC tCtrCmd5FC = { 0 };
    unsigned char ucCanMsgSrc[8] = { 0 };

    tCtrCmd5FC.uiRbtSlfChkReq = tRbtSlfChkCtrRq->uiRbtSlfChkReq;

    memcpy(ucCanMsgSrc, &tCtrCmd5FC, CAN_MSG_LEN);
    CharReverse(ucCanMsgSrc,8);
    memcpy(tCanMsg->DATA, ucCanMsgSrc, CAN_MSG_LEN);
    tCanMsg->ID = robot::common::RobotSelfCheckCtrReq;
    tCanMsg->LEN = 8;
    tCanMsg->MSGTYPE = MSGTYPE_STANDARD;
}

//机器人手动控制命令--数据组包
void MessageCoder::CanMsgPackageID_5FD(T_RBT_DRIVER_CTR * tRbtDriverCtr, TPCANMsg *tCanMsg)
{
    T_CONTROL_CMD_5FD tCtrCmd5FD = { 0 };
    unsigned char ucCanMsgSrc[8] = { 0 };

    tCtrCmd5FD.uiAccMotCtrl = tRbtDriverCtr->uiAccMotCtrl;
    tCtrCmd5FD.uiBrkMotCtrl = tRbtDriverCtr->uiBrkMotCtrl;
    tCtrCmd5FD.uiClchMotCtrl = tRbtDriverCtr->uiClchMotCtrl;
    tCtrCmd5FD.uiMotMove = tRbtDriverCtr->uiMotMove;
    tCtrCmd5FD.uiStrMotCtrl = tRbtDriverCtr->uiStrMotCtrl;
    tCtrCmd5FD.uiXSftMotCtrl = tRbtDriverCtr->uiXSftMotCtrl;
    tCtrCmd5FD.uiYSftMotCtrl = tRbtDriverCtr->uiYSftMotCtrl;

    memcpy(ucCanMsgSrc, &tCtrCmd5FD, CAN_MSG_LEN);
    CharReverse(ucCanMsgSrc,8);
    memcpy(tCanMsg->DATA, ucCanMsgSrc, CAN_MSG_LEN);
    tCanMsg->ID = robot::common::RobotDriverCtr;
    tCanMsg->LEN = 8;
    tCanMsg->MSGTYPE = MSGTYPE_STANDARD;
}

//设备管理--数据组包
void MessageCoder::CanMsgPackageID_7F6(T_RBT_MANAGER * tRbtManager, TPCANMsg *tCanMsg)
{
    T_CONTROL_CMD_7F6 tCtrCmd7F6 = { 0 };
    unsigned char ucCanMsgSrc[8] = { 0 };

    tCtrCmd7F6.uiAccMotInt = tRbtManager->uiAccMotInt;
    tCtrCmd7F6.uiBrkMotInt = tRbtManager->uiBrkMotInt;
    tCtrCmd7F6.uiClchMotInt = tRbtManager->uiClchMotInt;
    tCtrCmd7F6.uiFltClr = tRbtManager->uiFltClr;
    tCtrCmd7F6.uiStrMotInt = tRbtManager->uiStrMotInt;
    tCtrCmd7F6.uiXSftMotInt = tRbtManager->uiXSftMotInt;
    tCtrCmd7F6.uiYSftMotInt = tRbtManager->uiYSftMotInt;
    tCtrCmd7F6.uiZSftMotInt = tRbtManager->uiZSftMotInt;

    memcpy(ucCanMsgSrc, &tCtrCmd7F6, CAN_MSG_LEN);
    CharReverse(ucCanMsgSrc,8);
    memcpy(tCanMsg->DATA, ucCanMsgSrc, CAN_MSG_LEN);
    tCanMsg->ID = robot::common::RobotManager;
    tCanMsg->LEN = 8;
    tCanMsg->MSGTYPE = MSGTYPE_STANDARD;
}

//上位机监控--数据组包
void MessageCoder::CanMsgPackageID_7F8(T_ICU_MONITOR * tIcuMonitor, TPCANMsg *tCanMsg)
{
    T_CONTROL_CMD_7F8 tCtrCmd7F8 = { 0 };
    unsigned char ucCanMsgSrc[8] = { 0 };

    tCtrCmd7F8.uiICUAlive = tIcuMonitor->uiICUAlive;
    tCtrCmd7F8.uiICUClsReq = tIcuMonitor->uiICUClsReq;

    memcpy(ucCanMsgSrc, &tCtrCmd7F8, CAN_MSG_LEN);
    CharReverse(ucCanMsgSrc,8);
    memcpy(tCanMsg->DATA, ucCanMsgSrc, CAN_MSG_LEN);
    tCanMsg->ID = robot::common::ICUMonitor;
    tCanMsg->LEN = 8;
    tCanMsg->MSGTYPE = MSGTYPE_STANDARD;
}

//执行机构加速度控制请求--数据组包
void MessageCoder::CanMsgPackageID_5F5(T_STR_ACC_CTR_REQ * tActAccCtrReq, TPCANMsg *tCanMsg)
{
    T_CONTROL_CMD_5F5 tCtrCmd5F5 = { 0 };
    unsigned char ucCanMsgSrc[8] = { 0 };

    tCtrCmd5F5.uiStrDecCtrReq = tActAccCtrReq->uiStrDecCtrReq;
    tCtrCmd5F5.uiStrAccCtrReq = tActAccCtrReq->uiStrAccCtrReq;

    memcpy(ucCanMsgSrc, &tCtrCmd5F5, CAN_MSG_LEN);
    CharReverse(ucCanMsgSrc,8);
    memcpy(tCanMsg->DATA, ucCanMsgSrc, CAN_MSG_LEN);
    tCanMsg->ID = robot::common::ActuatorAccelerateCtrReq;
    tCanMsg->LEN = 8;
    tCanMsg->MSGTYPE = MSGTYPE_STANDARD;
}

//执行机构限值设置--数据组包
void MessageCoder::CanMsgPackageID_7F3(T_ACT_CTR_LIMT * tActCtrLimt, TPCANMsg *tCanMsg)
{
    T_CONTROL_CMD_7F3 tCtrCmd7F3 = { 0 };
    unsigned char ucCanMsgSrc[8] = { 0 };

    tCtrCmd7F3.iStrMaxLimt = tActCtrLimt->iStrMaxLimt;
    tCtrCmd7F3.uiAccMaxLimt = tActCtrLimt->uiAccMaxLimt;
    tCtrCmd7F3.uiBrkMaxLimt = tActCtrLimt->uiBrkMaxLimt;
    tCtrCmd7F3.uiSftType = tActCtrLimt->uiSftType;

    memcpy(ucCanMsgSrc, &tCtrCmd7F3, CAN_MSG_LEN);
    CharReverse(ucCanMsgSrc,8);
    memcpy(tCanMsg->DATA, ucCanMsgSrc, CAN_MSG_LEN);
    tCanMsg->ID = robot::common::ActuatorCtrLimit;
    tCanMsg->LEN = 8;
    tCanMsg->MSGTYPE = MSGTYPE_STANDARD;

}

//力矩保护设置--数据组包
void MessageCoder::CanMsgPackageID_7F4(T_TORQUE_PROTECT_SETUP * tTrqPrtctSetp, TPCANMsg *tCanMsg)
{
    T_CONTROL_CMD_7F4 tCtrCmd7F4 = { 0 };
    unsigned char ucCanMsgSrc[8] = { 0 };

    tCtrCmd7F4.uiStrTorqLimit = tTrqPrtctSetp->uiStrTorqLimit;
    tCtrCmd7F4.uiStrTorqSwitch = tTrqPrtctSetp->uiStrTorqSwitch;

    memcpy(ucCanMsgSrc, &tCtrCmd7F4, CAN_MSG_LEN);
    CharReverse(ucCanMsgSrc,8);
    memcpy(tCanMsg->DATA, ucCanMsgSrc, CAN_MSG_LEN);
    tCanMsg->ID = robot::common::TorqueProtectStp;
    tCanMsg->LEN = 8;
    tCanMsg->MSGTYPE = MSGTYPE_STANDARD;
}

//制动机构时间控制设置--数据组包
void MessageCoder::CanMsgPackageID_5F1(T_BRK_FORCE_CTR_SETUP * tBrkFrcCtrStp, TPCANMsg *tCanMsg)
{
    T_CONTROL_CMD_5F1 tCtrCmd5F1 = { 0 };
    unsigned char ucCanMsgSrc[8] = { 0 };

    T_CONTROL_CMD_5F1_UNION tCtrCmd5F1Union;
    memset(&tCtrCmd5F1Union, 0, sizeof(T_CONTROL_CMD_5F1_UNION));
    tCtrCmd5F1Union.tCtrCmd5F1Dest.uiBscCtr = tBrkFrcCtrStp->uiBscCtr;

    tCtrCmd5F1.iCtrSpd = tBrkFrcCtrStp->iCtrSpd;
    tCtrCmd5F1.uiBrkCtrWay = tBrkFrcCtrStp->uiBrkCtrWay;
    tCtrCmd5F1.uiBrkTm = tBrkFrcCtrStp->uiBrkTm;
    tCtrCmd5F1.uiBscCtrH = tCtrCmd5F1Union.tCtrCmd5F1Src.uiBscCtrH;
    tCtrCmd5F1.uiBscCtrL = tCtrCmd5F1Union.tCtrCmd5F1Src.uiBscCtrL;

    memcpy(ucCanMsgSrc, &tCtrCmd5F1, CAN_MSG_LEN);
    CharReverse(ucCanMsgSrc,8);
    memcpy(tCanMsg->DATA, ucCanMsgSrc, CAN_MSG_LEN);
    tCanMsg->ID = robot::common::BrkTimCtrlStp;
    tCanMsg->LEN = 8;
    tCanMsg->MSGTYPE = MSGTYPE_STANDARD;
}

//转向三角波--数据组包
void MessageCoder::CanMsgPackageID_5F6(T_STR_TRIANGLE_LINE_CTR * tStrTrnglLnCtr, TPCANMsg *tCanMsg)
{
    T_CONTROL_CMD_5F6 tCtrCmd5F6 = { 0 };
    unsigned char ucCanMsgSrc[8] = { 0 };

    T_CONTROL_CMD_5F6_UNION tControlCmd5F6Union;
    memset(&tControlCmd5F6Union, 0, sizeof(T_CONTROL_CMD_5F6_UNION));
    tControlCmd5F6Union.tCtrCmd5F6Dest.iStrSpdk = tStrTrnglLnCtr->iStrSpdk;

    tCtrCmd5F6.uiStrLT = tStrTrnglLnCtr->uiStrLT;
    tCtrCmd5F6.uiStrNum = tStrTrnglLnCtr->uiStrNum;
    tCtrCmd5F6.iStrPstBsc = tStrTrnglLnCtr->iStrPstBsc;
    tCtrCmd5F6.iStrSpdkH = tControlCmd5F6Union.tCtrCmd5F6Src.iStrSpdkH;
    tCtrCmd5F6.iStrSpdkL = tControlCmd5F6Union.tCtrCmd5F6Src.iStrSpdkL;
    tCtrCmd5F6.iStrTA = tStrTrnglLnCtr->iStrTA;
    tCtrCmd5F6.uiStrTT = tStrTrnglLnCtr->uiStrTT;

    memcpy(ucCanMsgSrc, &tCtrCmd5F6, CAN_MSG_LEN);
    CharReverse(ucCanMsgSrc,8);
    memcpy(tCanMsg->DATA, ucCanMsgSrc, CAN_MSG_LEN);
    tCanMsg->ID = robot::common::SteerTranLineCtr;
    tCanMsg->LEN = 8;
    tCanMsg->MSGTYPE = MSGTYPE_STANDARD;
}

//转向正弦控制--数据组包
void MessageCoder::CanMsgPackageID_5F7(T_STR_SIN_CTR * tStrSinCtr, TPCANMsg *tCanMsg)
{
    T_CONTROL_CMD_5F7 tCtrCmd5F7 = { 0 };
    unsigned char ucCanMsgSrc[8] = { 0 };

    tCtrCmd5F7.uiStrT = tStrSinCtr->uiStrT;
    tCtrCmd5F7.uiStrA = tStrSinCtr->uiStrA;
    tCtrCmd5F7.uiStrDt = tStrSinCtr->uiStrDt;
    tCtrCmd5F7.iStrFi = tStrSinCtr->iStrFi;
    tCtrCmd5F7.uiStrFrtb = tStrSinCtr->uiStrFrtb;
    tCtrCmd5F7.uiStrFrtk = tStrSinCtr->uiStrFrtk;

    memcpy(ucCanMsgSrc, &tCtrCmd5F7, CAN_MSG_LEN);
    CharReverse(ucCanMsgSrc,8);
    memcpy(tCanMsg->DATA, ucCanMsgSrc, CAN_MSG_LEN);
    tCanMsg->ID = robot::common::SteerSinCtr;
    tCanMsg->LEN = 8;
    tCanMsg->MSGTYPE = MSGTYPE_STANDARD;
}

//油门机构位置时间控制--数据组包
void MessageCoder::CanMsgPackageID_5F3(T_ACC_POSITION_CTR_SETUP * tAccPstnCtrStp, TPCANMsg *tCanMsg)
{
    T_CONTROL_CMD_5F3 tCtrCmd5F3 = { 0 };
    unsigned char ucCanMsgSrc[8] = { 0 };

    T_CONTROL_CMD_5F3_UNION tControlCmd5F3Union;
    memset(&tControlCmd5F3Union, 0, sizeof(T_CONTROL_CMD_5F3_UNION));
    tControlCmd5F3Union.tCtrCmd5F3Dest.uiAccPstBsc = tAccPstnCtrStp->uiAccPstBsc;

    tCtrCmd5F3.iAccSpdP = tAccPstnCtrStp->iAccSpdP;
    tCtrCmd5F3.uiAccPT = tAccPstnCtrStp->uiAccPT;
    tCtrCmd5F3.uiAccPstBscH = tControlCmd5F3Union.tCtrCmd5F3Src.uiAccPstBscH;
    tCtrCmd5F3.uiAccPstBscL = tControlCmd5F3Union.tCtrCmd5F3Src.uiAccPstBscL;

    memcpy(ucCanMsgSrc, &tCtrCmd5F3, CAN_MSG_LEN);
    CharReverse(ucCanMsgSrc,8);
    memcpy(tCanMsg->DATA, ucCanMsgSrc, CAN_MSG_LEN);
    tCanMsg->ID = robot::common::AccPositionCtrStp;
    tCanMsg->LEN = 8;
    tCanMsg->MSGTYPE = MSGTYPE_STANDARD;
}

//方向盘位置时间序列控制--数据组包
void MessageCoder::CanMsgPackageID_5F8(T_STR_POSITION_TIME_CTR * tStrPstnTmCtr, TPCANMsg *tCanMsg)
{
    T_CONTROL_CMD_5F8 tCtrCmd5F8 = { 0 };
    unsigned char ucCanMsgSrc[8] = { 0 };

    T_CONTROL_CMD_5F8_UNION tCtrCmd5F8Union;
    memset(&tCtrCmd5F8Union, 0, sizeof(T_CONTROL_CMD_5F8_UNION));
    tCtrCmd5F8Union.tCtrCmd5F8Dest.uiRelTime = tStrPstnTmCtr->uiRelTime;

    tCtrCmd5F8.uiNum = tStrPstnTmCtr->uiNum;
    tCtrCmd5F8.uiRelTimeH = tCtrCmd5F8Union.tCtrCmd5F8Src.uiRelTimeH;
    tCtrCmd5F8.uiRelTimeL = tCtrCmd5F8Union.tCtrCmd5F8Src.uiRelTimeL;
    tCtrCmd5F8.uiSequenceID = tStrPstnTmCtr->uiSequenceID;
    tCtrCmd5F8.iStrAngReq = tStrPstnTmCtr->iStrAngReq;
    tCtrCmd5F8.uiTotalNum = tStrPstnTmCtr->uiTotalNum;

    memcpy(ucCanMsgSrc, &tCtrCmd5F8, CAN_MSG_LEN);
    CharReverse(ucCanMsgSrc,8);
    memcpy(tCanMsg->DATA, ucCanMsgSrc, CAN_MSG_LEN);
    tCanMsg->ID = robot::common::SteerPositionTimeCtr;
    tCanMsg->LEN = 8;
    tCanMsg->MSGTYPE = MSGTYPE_STANDARD;
}

//车辆一键启动--数据组包
void MessageCoder::CanMsgPackageID_5FB(T_VEHICLE_START * tVehicleStart, TPCANMsg *tCanMsg)
{
    T_CONTROL_CMD_5FB tCtrCmd5FB = { 0 };
    unsigned char ucCanMsgSrc[8] = { 0 };

    tCtrCmd5FB.uiVehSwReq = tVehicleStart->uiVehSwReq;

    memcpy(ucCanMsgSrc, &tCtrCmd5FB, CAN_MSG_LEN);
    CharReverse(ucCanMsgSrc,8);
    memcpy(tCanMsg->DATA, ucCanMsgSrc, CAN_MSG_LEN);
    tCanMsg->ID = robot::common::VehicleStart;
    tCanMsg->LEN = 8;
    tCanMsg->MSGTYPE = MSGTYPE_STANDARD;
}

//坡度模板--数据组包
void MessageCoder::CanMsgPackageID_5FE(T_SLOPE_MODE *tSlopeMODE,TPCANMsg *tCanMsg)
{
    T_CONTROL_CMD_5FE tCtrCmd5FE = {0};
    unsigned char ucCanMsgSrc[8] = {0};

    tCtrCmd5FE.iSlopeRequest = tSlopeMODE->iSlopeRequest;

    memcpy(ucCanMsgSrc, &tCtrCmd5FE, CAN_MSG_LEN);
    CharReverse(ucCanMsgSrc,8);
    memcpy(tCanMsg->DATA, ucCanMsgSrc, CAN_MSG_LEN);
    tCanMsg->ID = robot::common::SlopeMODE;
    tCanMsg->LEN = 8;
    tCanMsg->MSGTYPE = MSGTYPE_STANDARD;
}


//车辆状态1--数据组包
void MessageCoder::CanMsgPackageID_6F1(T_VEHICLE_STATUS_1 *tVehicleStatus1, TPCANMsg *tCanMsg)
{
    T_STATE_IN_6F1 tStateIn6F1 = { 0 };
    unsigned char ucCanMsgSrc[8] = { 0 };

    tStateIn6F1.uiAccPsng = tVehicleStatus1->uiAccPsng;
    tStateIn6F1.uiBrkPsng = tVehicleStatus1->uiBrkPsng;
    tStateIn6F1.uiClchPsng = tVehicleStatus1->uiClchPsng;
    tStateIn6F1.uiDrvSftTrgtGear = tVehicleStatus1->uiDrvSftTrgtGear;
    tStateIn6F1.uiSftPsng = tVehicleStatus1->uiSftPsng;
    tStateIn6F1.uiVehHBrkStat = tVehicleStatus1->uiVehHBrkStat;
    tStateIn6F1.uiVehSftMd = tVehicleStatus1->uiVehSftMd;
    tStateIn6F1.uiVehSwStat = tVehicleStatus1->uiVehSwStat;

    memcpy(ucCanMsgSrc, &tStateIn6F1, CAN_MSG_LEN);
    CharReverse(ucCanMsgSrc,8);
    memcpy(tCanMsg->DATA, ucCanMsgSrc, CAN_MSG_LEN);
    tCanMsg->ID = robot::common::VehicleStatus1;
    tCanMsg->LEN = 8;
    tCanMsg->MSGTYPE = MSGTYPE_STANDARD;
}

//车辆状态2--数据组包
void MessageCoder::CanMsgPackageID_6F2(T_VEHICLE_STATUS_2 *tVehicleStatus2, TPCANMsg *tCanMsg)
{
    T_STATE_IN_6F2 tStateIn6F2 = { 0 };
    unsigned char ucCanMsgSrc[8] = { 0 };

    T_STATE_IN_6F2_UNION tStateIn6F2Union;
    memset(&tStateIn6F2Union, 0, sizeof(T_STATE_IN_6F2_UNION));
    tStateIn6F2Union.tStateIn6F2Dest.iStrAngGrd = tVehicleStatus2->iStrAngGrd;

    tStateIn6F2.iStrAng = tVehicleStatus2->iStrAng;
    tStateIn6F2.iStrAngGrdH = tStateIn6F2Union.tStateIn6F2Src.iStrAngGrdH;
    tStateIn6F2.iStrAngGrdL = tStateIn6F2Union.tStateIn6F2Src.iStrAngGrdL;
    tStateIn6F2.uiEngSpd = tVehicleStatus2->uiEngSpd;
    tStateIn6F2.uiVehSpd = tVehicleStatus2->uiVehSpd;

    memcpy(ucCanMsgSrc, &tStateIn6F2, CAN_MSG_LEN);
        CharReverse(ucCanMsgSrc,8);
    memcpy(tCanMsg->DATA, ucCanMsgSrc, CAN_MSG_LEN);
    tCanMsg->ID = robot::common::VehicleStatus2;
    tCanMsg->LEN = 8;
    tCanMsg->MSGTYPE = MSGTYPE_STANDARD;
}

//车辆状态3--数据组包
void MessageCoder::CanMsgPackageID_6F3(T_VEHICLE_STATUS_3 *tVehicleStatus3, TPCANMsg *tCanMsg)
{
    T_STATE_IN_6F3 tStateIn6F3 = { 0 };
    unsigned char ucCanMsgSrc[8] = { 0 };

    tStateIn6F3.uiVehBatCur = tVehicleStatus3->uiVehBatCur;
    tStateIn6F3.uiVehBatQut = tVehicleStatus3->uiVehBatQut;
    tStateIn6F3.uiVehBatVlt = tVehicleStatus3->uiVehBatVlt;

    memcpy(ucCanMsgSrc, &tStateIn6F3, CAN_MSG_LEN);
    CharReverse(ucCanMsgSrc,8);
    memcpy(tCanMsg->DATA, ucCanMsgSrc, CAN_MSG_LEN);
    tCanMsg->ID = robot::common::VehicleStatus3;
    tCanMsg->LEN = 8;
    tCanMsg->MSGTYPE = MSGTYPE_STANDARD;
}

void MessageCoder::CanMsgPackageID_6E2 (T_STR_LRN_PSNG *tStrLrnPsng,TPCANMsg *tCanMsg)
{
  T_STATE_OUT_6E2 tStateIn6E2 = { 0 };
  unsigned char ucCanMsgSrc[8] = { 0 };

  T_STATE_OUT_6E2_UNION tStateIn6E2Union;
  memset(&tStateIn6E2Union, 0, sizeof(T_STATE_OUT_6E2_UNION));
  tStateIn6E2Union.tStateOut6E2Dest.iClbtStrAngLftMax = tStrLrnPsng->iClbtStrAngLftMax;

  tStateIn6E2.iClbtStrAng0 = tStrLrnPsng->iClbtStrAng0;
  tStateIn6E2.iClbtStrAngLftMaxH = tStateIn6E2Union.tStateOut6E2Src.iClbtStrAngLftMaxH;
  tStateIn6E2.iClbtStrAngLftMaxL = tStateIn6E2Union.tStateOut6E2Src.iClbtStrAngLftMaxL;
  tStateIn6E2.iClbtStrAngRtMax = tStrLrnPsng->iClbtStrAngRtMax;

  memcpy(ucCanMsgSrc, &tStateIn6E2, CAN_MSG_LEN);
      CharReverse(ucCanMsgSrc,8);
  memcpy(tCanMsg->DATA, ucCanMsgSrc, CAN_MSG_LEN);
  tCanMsg->ID = robot::common::SteerLearnPosBack;
  tCanMsg->LEN = 8;
  tCanMsg->MSGTYPE = MSGTYPE_STANDARD;
}


void MessageCoder::CanMsgPackageID_201 (T_ESTP_CTRL *tEstpCtrl,TPCANMsg *tCanMsg)
{
  T_CONTROL_CMD_201 tControlCmd201 = { 0 };
  unsigned char ucCanMsgSrc[8] = { 0 };

  tControlCmd201.uiEstpEnReq = tEstpCtrl->uiEstpEnReq;
  tControlCmd201.uiPauseEnReq = tEstpCtrl->uiPauseEnReq;
  tControlCmd201.uiPmpEnReq = tEstpCtrl->uiPmpEnReq;

  memcpy(ucCanMsgSrc, &tControlCmd201, CAN_MSG_LEN);
      CharReverse(ucCanMsgSrc,8);
  memcpy(tCanMsg->DATA, ucCanMsgSrc, CAN_MSG_LEN);
  tCanMsg->ID = robot::common::EstpCtrl;
  tCanMsg->LEN = 8;
  tCanMsg->MSGTYPE = MSGTYPE_STANDARD;
}

void MessageCoder::CanMsgPackageID_7F0 (T_WORKING_MODE_CONFING *tWorkingModeConfing,TPCANMsg *tCanMsg)
{
  T_CONTROL_CMD_7F0 tControlCmd7F0 = { 0 };
  unsigned char ucCanMsgSrc[8] = { 0 };

  tControlCmd7F0.uiOnlineOffline = tWorkingModeConfing->uiOnlineOffline;
  tControlCmd7F0.uiRbtAplySenro = tWorkingModeConfing->uiRbtAplySenro;

  memcpy(ucCanMsgSrc, &tControlCmd7F0, CAN_MSG_LEN);
      CharReverse(ucCanMsgSrc,8);
  memcpy(tCanMsg->DATA, ucCanMsgSrc, CAN_MSG_LEN);
  tCanMsg->ID = robot::common::RobotWorkMd;
  tCanMsg->LEN = 8;
  tCanMsg->MSGTYPE = MSGTYPE_STANDARD;
}

void MessageCoder::CanMsgPackageID_7F1 (T_CONTROL_MODE *tControlMode,TPCANMsg *tCanMsg)
{
  T_CONTROL_CMD_7F1 tControlCmd7F1 = { 0 };
  unsigned char ucCanMsgSrc[8] = { 0 };

  tControlCmd7F1.uiCtrMdReq = tControlMode->uiCtrMdReq;

  memcpy(ucCanMsgSrc, &tControlCmd7F1, CAN_MSG_LEN);
      CharReverse(ucCanMsgSrc,8);
  memcpy(tCanMsg->DATA, ucCanMsgSrc, CAN_MSG_LEN);
  tCanMsg->ID = robot::common::SysCtrlMode;
  tCanMsg->LEN = 8;
  tCanMsg->MSGTYPE = MSGTYPE_STANDARD;
}

void MessageCoder::CanMsgPackageID_5FF (T_IO_REQ *tIOReq,TPCANMsg *tCanMsg)
{
  T_CONTROL_CMD_5FF tControlCmd5FF = { 0 };
  unsigned char ucCanMsgSrc[8] = { 0 };

  tControlCmd5FF.IO_0Req = tIOReq->IO_0Req;
  tControlCmd5FF.IO_1Req = tIOReq->IO_1Req;

  memcpy(ucCanMsgSrc, &tControlCmd5FF, CAN_MSG_LEN);
      CharReverse(ucCanMsgSrc,8);
  memcpy(tCanMsg->DATA, ucCanMsgSrc, CAN_MSG_LEN);
  tCanMsg->ID = robot::common::IOMode;
  tCanMsg->LEN = 8;
  tCanMsg->MSGTYPE = MSGTYPE_STANDARD;
}

//数据解析
void * MessageCoder::CanMsgAnalyse(int id,unsigned char * ucData)
{
    void *p = nullptr;
    switch (id)
    {
    case robot::common::ActuatorEnableReq:
        T_ACTUATOR_ENABLE_REQUEST tActEnableReq;
        CanMsgAnalyseID_7F5(ucData, &tActEnableReq);
        p = &tActEnableReq;
        break;
    case robot::common::SteerActCtrReq:
        T_STR_ACT_CTR_REQ tStrActCtrReq;
        CanMsgAnalyseID_5F4(ucData,&tStrActCtrReq);
        p = &tStrActCtrReq;
        break;
    case robot::common::VehicleStatus1://车辆状态1
        T_VEHICLE_STATUS_1 tVehicleStatus1;
        CanMsgAnalyseID_6F1(ucData,&tVehicleStatus1);
        p = &tVehicleStatus1;
        break;
    case robot::common::VehicleStatus2://车辆状态2
        T_VEHICLE_STATUS_2 tVehicleStatus2;
        CanMsgAnalyseID_6F2(ucData,&tVehicleStatus2);
        p = &tVehicleStatus2;
        break;
    case robot::common::VehicleStatus3://车辆状态3
        T_VEHICLE_STATUS_3 tVehicleStatus3;
        CanMsgAnalyseID_6F3(ucData, &tVehicleStatus3);
        p = &tVehicleStatus3;
        break;
    case robot::common::ComputerStaus://工控机状态
        T_COMPUTER_STATUS tComputerStat;
        CanMsgAnalyseID_6F4(ucData,&tComputerStat);
        p = &tComputerStat;
        break;
    case robot::common::ZSftStaus: //档位 Z 轴状态反馈
      T_ZSFT_STATUS tZSftStatus;
      CanMsgAnalyseID_6F5(ucData,&tZSftStatus);
      p = &tZSftStatus;
    case robot::common::RobotAcuatorPos://机器人执行机构实际位置
        T_RBT_ACT_PSNG tRbtActPsng ;
        CanMsgAnalyseID_6E0(ucData, &tRbtActPsng);
        p = &tRbtActPsng;
        break;
    case robot::common::PedalLearnPosBack://踏板学习位置反馈
        T_ACC_BRK_PSNG tAccBrkPsng;
        CanMsgAnalyseID_6E1(ucData,&tAccBrkPsng);
        p = &tAccBrkPsng;
        break;
    case robot::common::SteerLearnPosBack://转向学习位置反馈
        T_STR_LRN_PSNG tStrLrnPsng;
        CanMsgAnalyseID_6E2(ucData,&tStrLrnPsng);
        p = &tStrLrnPsng;
        break;
    case robot::common::ShiftLearnPosBack1://档位学习位置反馈1
        T_SFT_LRN_PST_1 tSftLrnPst1 ;
        CanMsgAnalyseID_6E3(ucData,&tSftLrnPst1);
        p = &tSftLrnPst1;
        break;
    case robot::common::CtrPowerStatus://控制器上电状态
        T_CTR_POW_STATUS tCtrPowStatus;
        CanMsgAnalyseID_6E4(ucData,&tCtrPowStatus);
        p = &tCtrPowStatus;
        break;
    case robot::common::ActuatorEnableStatus://执行机构使能状态
        T_ACT_ENABLE_STATUS tActEnableStatus;
        CanMsgAnalyseID_6E5(ucData,&tActEnableStatus);
        p = &tActEnableStatus;
        break;
    case robot::common::AccPedalStatusBack://油门控制状态反馈
        T_ACC_CTR_STATUS tAccCtrStatus ;
        CanMsgAnalyseID_6E6(ucData,&tAccCtrStatus);
        p = &tAccCtrStatus;
        break;
    case robot::common::BrkPedalStatusBack://制动踏板控制状态反馈
        T_BRK_CTR_STATUS tBrkCtrStatus ;
        CanMsgAnalyseID_6E7(ucData,&tBrkCtrStatus);
        p = &tBrkCtrStatus;
        break;
    case robot::common::ClutchPedalStatusBack://离合踏板控制状态反馈
        T_CLCH_CTR_STATUS tClchCtrStatus ;
        CanMsgAnalyseID_6E8(ucData,&tClchCtrStatus);
        p = &tClchCtrStatus;
        break;
    case robot::common::SteerStatusBack://转向控制状态反馈
        T_STR_STATUS tStrStatus;
        CanMsgAnalyseID_6E9(ucData, &tStrStatus);
        p = &tStrStatus;
        break;
    case robot::common::ShiftStatusBack://档位控制状态反馈
        T_SFT_STATUS tSftStatus ;
        CanMsgAnalyseID_6EA(ucData,&tSftStatus);
        p = &tSftStatus;
        break;
    case robot::common::RSysInfoBack1://系统运行状态参数反馈1
        T_SYS_INFO_1 tSysInfo1;
        CanMsgAnalyseID_6EB(ucData,&tSysInfo1);
        p = &tSysInfo1;
        break;
    case robot::common::RSysInfoBack2://系统运行状态参数反馈2
        T_SYS_INFO_2 tSysInfo2;
        CanMsgAnalyseID_6ED(ucData,&tSysInfo2);
        p = &tSysInfo2;
        break;
    case robot::common::SysLearnBack://系统学习状态反馈
        T_SYS_LRN tSysLrn;
        CanMsgAnalyseID_6EE(ucData,&tSysLrn);
        p = &tSysLrn;
        break;
    case robot::common::ShiftLearnPosBack2://档位学习位置反馈2
        T_SFT_LRN_PST_2 tSFtLrnPst2 ;
        CanMsgAnalyseID_6EF(ucData,&tSFtLrnPst2);
        p = &tSFtLrnPst2;
        break;
    case robot::common::SysMonitorBack://系统监控状态反馈
        T_SYS_MONITOR tSysMonitor;
        CanMsgAnalyseID_6D0(ucData,&tSysMonitor);
        p = &tSysMonitor;
        break;
    case robot::common::SysWarnErr://系统报警
        T_SYS_ERR tSysErr;
        CanMsgAnalyseID_6D1(ucData,&tSysErr);
        p = &tSysErr;
        break;
    case robot::common::SteerPosTimeReq://方向盘位置时间控制请求
        T_STR_PST_TIME_REQ tSteerPstTimeReq ;
        CanMsgAnalyseID_6D7(ucData,&tSteerPstTimeReq);
        p = &tSteerPstTimeReq;
        break;
    case robot::common::EstopInfo://Estop系统信息
        T_ESTOP_INFO tEstopInfo ;
        CanMsgAnalyseID_2E0(ucData,&tEstopInfo);
        p = &tEstopInfo;
        break;
    case robot::common::EstopErr://Estop故障码
        T_ESTOP_ERR tEstopErr;
        CanMsgAnalyseID_2E1(ucData,&tEstopErr);
        p = &tEstopErr;
        break;
    case robot::common::AccErr://油门(acc)电机报警
        T_MOT_CTR_ERR tMotCtrErrAcc ;
        CanMsgAnalyseErr(ucData,&tMotCtrErrAcc);
        p = &tMotCtrErrAcc;
        break;
    case robot::common::BrkErr://制动(brk)电机报警
        T_MOT_CTR_ERR tMotCtrErrBrk ;
        CanMsgAnalyseErr(ucData, &tMotCtrErrBrk);
        p = &tMotCtrErrBrk;
        break;
    case robot::common::XsftErr://挡位(sft)X电机报警
        T_MOT_CTR_ERR tMotCtrErrX ;
        CanMsgAnalyseErr(ucData, &tMotCtrErrX);
        p = &tMotCtrErrX;
        break;
    case robot::common::YsftErr://挡位(sft)Y电机报警
        T_MOT_CTR_ERR tMotCtrErrY ;
        CanMsgAnalyseErr(ucData, &tMotCtrErrY);
        p = &tMotCtrErrY;
        break;
    case robot::common::StrErr://转向(str)电机报警
        T_MOT_CTR_ERR tMotCtrErrStr ;
        CanMsgAnalyseErr(ucData, &tMotCtrErrStr);
        p = &tMotCtrErrStr;
        break;
    case robot::common::ObdAccPsng:
      T_OBD_ACC_PSNG tObdAccPsng ;
      OBDAnalyseID_070(ucData,&tObdAccPsng);
      p = &tObdAccPsng;
      break;
    default:
        break;
    }
    return p;
}

//数据组包
void MessageCoder::CanMsgPackage(void * p, int id, TPCANMsg *tCanMsg)
{
    switch (id)
    {
    case robot::common::PowerOnReq://控制器上电命令
        CanMsgPackageID_7F2((T_POWER_ON_REQUEST*)p, tCanMsg);
        break;
    case robot::common::ActuatorEnableReq://执行机构使能请求
        CanMsgPackageID_7F5((T_ACTUATOR_ENABLE_REQUEST*)p, tCanMsg);
        break;
    case robot::common::LearnModeReq://学习模式请求确认
        CanMsgPackageID_7F7((T_LEARN_MODE_REQUEST*)p,tCanMsg);
        break;
    case robot::common::AccPedalActCtrReq://油门踏板执行机构控制命令
        CanMsgPackageID_5F2((T_ACC_PEDAL_ACT_CTR_REQ*)p, tCanMsg);
        break;
    case robot::common::BrkPedalActCtrReq://制动踏板执行机构控制命令
        CanMsgPackageID_5F0((T_BRK_PEDAL_ACT_CTR_REQ*)p, tCanMsg);
        break;
    case robot::common::SteerActCtrReq://转向执行机构控制命令
        CanMsgPackageID_5F4((T_STR_ACT_CTR_REQ*)p, tCanMsg);
        break;
    case robot::common::ShiftActCtrReq://档位执行机构控制命令
        CanMsgPackageID_5FA((T_SFT_ACT_CTR_REQ*)p,tCanMsg);
        break;
    case robot::common::RobotSelfCheckCtrReq://机器人自检控制命令
        CanMsgPackageID_5FC((T_RBT_SLF_CHK_CTR_REQ *)p, tCanMsg);
        break;
    case robot::common::RobotDriverCtr://机器人手动控制命令
        CanMsgPackageID_5FD((T_RBT_DRIVER_CTR *)p, tCanMsg);
        break;
    case robot::common::RobotManager://设备管理
        CanMsgPackageID_7F6((T_RBT_MANAGER *)p, tCanMsg);
        break;
    case robot::common::ICUMonitor://上位机监控
        CanMsgPackageID_7F8((T_ICU_MONITOR *)p, tCanMsg);
        break;
    case robot::common::ActuatorAccelerateCtrReq://转向机构加速度控制请求
        CanMsgPackageID_5F5((T_STR_ACC_CTR_REQ *)p,tCanMsg);
        break;
    case robot::common::ActuatorCtrLimit://执行机构限值设置
        CanMsgPackageID_7F3((T_ACT_CTR_LIMT *)p,tCanMsg);
        break;
    case robot::common::TorqueProtectStp://力矩保护设置
        CanMsgPackageID_7F4((T_TORQUE_PROTECT_SETUP *)p,tCanMsg);
        break;
    case robot::common::BrkTimCtrlStp://制动机构时间控制设置
        CanMsgPackageID_5F1((T_BRK_FORCE_CTR_SETUP *)p,tCanMsg);
        break;
    case robot::common::SteerTranLineCtr://转向三角波
        CanMsgPackageID_5F6((T_STR_TRIANGLE_LINE_CTR *)p,tCanMsg);
        break;
    case robot::common::SteerSinCtr://转向正弦控制
        CanMsgPackageID_5F7((T_STR_SIN_CTR *)p,tCanMsg);
        break;
    case robot::common::AccPositionCtrStp://油门机构位置时间控制
        CanMsgPackageID_5F3((T_ACC_POSITION_CTR_SETUP *)p,tCanMsg);
        break;
    case robot::common::SteerPositionTimeCtr://方向盘位置时间序列控制
        CanMsgPackageID_5F8((T_STR_POSITION_TIME_CTR *)p,tCanMsg);
        break;
    case robot::common::VehicleStart://车辆一键启动
        CanMsgPackageID_5FB((T_VEHICLE_START *)p,tCanMsg);
        break;
    case robot::common::SlopeMODE:  //坡度模板
        CanMsgPackageID_5FE((T_SLOPE_MODE *)p,tCanMsg);
        break;
    case robot::common::VehicleStatus1://车辆状态1
        CanMsgPackageID_6F1((T_VEHICLE_STATUS_1 *)p,tCanMsg);
        break;
    case robot::common::VehicleStatus2://车辆状态2
        CanMsgPackageID_6F2((T_VEHICLE_STATUS_2 *)p,tCanMsg);
        break;
    case robot::common::VehicleStatus3://车辆状态3
        CanMsgPackageID_6F3((T_VEHICLE_STATUS_3 *)p,tCanMsg);
        break;
    case robot::common::SteerLearnPosBack:
      CanMsgPackageID_6E2((T_STR_LRN_PSNG *)p,tCanMsg);
      break;
    case robot::common::EstpCtrl:
      CanMsgPackageID_201((T_ESTP_CTRL *)p,tCanMsg);
      break;
    case robot::common::RobotWorkMd:
      CanMsgPackageID_7F0((T_WORKING_MODE_CONFING *)p,tCanMsg);
      break;
    case robot::common::SysCtrlMode:
      CanMsgPackageID_7F1((T_CONTROL_MODE *)p,tCanMsg);
      break;
    case robot::common::IOMode:
      CanMsgPackageID_5FF((T_IO_REQ *)p,tCanMsg);
      break;
    default:
        break;
    }
}

void MessageCoder::ConvertUnCharToStr(char *str,unsigned char *UnChar,int ucLen)
{
    for(int i=0;i<ucLen;i++)
    {
      sprintf(str+i*2,"%02x",UnChar[i]);
    }
}

}// namespace canbus
}// namespace robot
