/******************************************************************************
 * Copyright 2020 The siasun Transport Authors. All Rights Reserved.
 * Author: madesheng
 * Licensed under the Apache License, Version 1.0 (the "License");
 *****************************************************************************/

#ifndef ROBOT_COMMON_ROBOT_APP_H_
#define ROBOT_COMMON_ROBOT_APP_H_

#include <csignal>
#include <string>
#include "gloghelper.h"
#include "ros/ros.h"


/**
 * @namespace robot::common
 * @brief robot::common
 */
namespace robot {
namespace common {

/**
 * 定义节点启动接口，所有节点统一继承该类
 */
class RobotApp {
 public:
  /**
   * 节点名称，用于标定每个节点
   */
  virtual std::string Name() const = 0;

  /**
   * 刷新函数
   */
  virtual int Spin();

  /**
   * 默认析构函数
   */
  virtual ~RobotApp() = default;


 protected:
  /**
   * 初始化，加载配置文件
   */
  virtual int Init() = 0;

  /**
   * 启动节点信号，发布，接收信息，响应定时函数
   */
  virtual int Start() = 0;

  /**
   *关闭节点信息
   */
  virtual void Stop() = 0;

  /**
   * @brief create a timer which will call a callback at the specified
   * rate. It takes a class member function, and a bare pointer to the
   * object to call the method on.
   */
  template <class T>
  ros::Timer CreateTimer(ros::Duration period,
                                void (T::*callback)(const ros::TimerEvent &),
                                T *obj, bool oneshot = false,
                                bool autostart = true) {
    return node_handle_->createTimer(period, callback, obj, oneshot,
                                                 autostart);
  }

 protected:
  // The node handler of ROS
  std::unique_ptr<ros::NodeHandle> node_handle_;

};

void robot_app_sigint_handler(int signal_num);

}  // namespace common
}  // namespace robot

#define ROBOT_MAIN(APP)                                                 \
  int main(int argc, char **argv) {                                     \
    signal(SIGINT, robot::common::robot_app_sigint_handler);            \
    APP robot_app_;                                                     \
    robot::common::GLogHelper glog(argv[0], robot_app_.Name());         \
    ros::init(argc, argv, robot_app_.Name());                           \
    robot_app_.Spin();                                                  \
    return 0;                                                           \
  }

#endif  // ROBOT_COMMON_ROBOT_APP_H_
