/******************************************************************************
 * Copyright 2020 The siasun Transport Authors. All Rights Reserved.
 * Author: zhangyu,Altered: madesheng
 * Licensed under the Apache License, Version 1.0 (the "License");
 *****************************************************************************/
#include "perception.h"
#include "../common/message.h"
#include "../common/canmsgobdmsgid.h"


namespace  robot {
namespace perception {

#define CAN_MSG_LEN 8


std::string Perception::Name() const { return "perception"; }

int Perception::Init()
{
    LOG(INFO) << Name() << " Init start...";

    node_handle_.reset(new ros::NodeHandle());

    LOG(INFO) << Name() << " Init end.";
    return ROBOT_SUCCESS;
}

int Perception::Start()
{
    LOG(INFO) << Name() << " Start start...";
    try
    {
      // set timer to triger publish info periodly
      timer_ = CreateTimer(ros::Duration(0.5), &Perception::OnTimer, this);
         sub = node_handle_->subscribe(robot::common::CanDataAnalysis, 1, &Perception::CanTopicCallback, this);

        bus_pub_6E5 = node_handle_->advertise<::perception::aesMsg>(robot::common::AesData, 1);
        bus_pub_6EE = node_handle_->advertise<::perception::slMsg>(robot::common::SlData, 1);
        bus_pub_6ED = node_handle_->advertise<::perception::si2Msg>(robot::common::Si2Data, 1);

        bus_pub_6D1 = node_handle_->advertise<::perception::seMsg>(robot::common::SeData, 1);

        bus_pub_6D2 = node_handle_->advertise<::perception::cecMsg2>(robot::common::AcceData, 1);
        bus_pub_6D3 = node_handle_->advertise<::perception::cecMsg2>(robot::common::BrkeData, 1);
        bus_pub_6D4 = node_handle_->advertise<::perception::cecMsg>(robot::common::XsfteData, 1);
        bus_pub_6D5 = node_handle_->advertise<::perception::cecMsg>(robot::common::YsfteData, 1);
        bus_pub_6D6 = node_handle_->advertise<::perception::cecMsg>(robot::common::StreData, 1);

        bus_pub_2E0 = node_handle_->advertise<::perception::eiMsg>(robot::common::EiData, 1);
        bus_pub_2E1 = node_handle_->advertise<::perception::eeMsg>(robot::common::EeData, 1);

        clientActuatorEnableRequest = node_handle_->serviceClient<canbus::ActuatorEnableRequest>(robot::common::ActuatorEnableService);

        bus_pub_6E4 = node_handle_->advertise<::perception::powerMsg>(robot::common::PowerData, 1);

        bus_pub_6E1 = node_handle_->advertise<::perception::pedLrnPosMsg>(robot::common::PedLrnPosData,1);
        bus_pub_6E2 = node_handle_->advertise<::perception::strLrnPosMsg>(robot::common::StrLrnPosData,1);
        bus_pub_6E3 = node_handle_->advertise<::perception::sfLrnPos1Msg>(robot::common::SfLrnPos1Data,1);
        bus_pub_6EF = node_handle_->advertise<::perception::sfLrnPos2Msg>(robot::common::SfLrnPos2Data,1);
        bus_pub_6EB = node_handle_->advertise<::perception::si1Msg>(robot::common::Si1Data,1);

        bus_pub_6D0 = node_handle_->advertise<::perception::smMsg>(robot::common::SmData,1);

        bus_pub_6F5 = node_handle_->advertise<::perception::zsftStatsMsg>(robot::common::ZSftData,1);

        ros::spin();
    }
    catch(ros::Exception e)
    {
         LOG(ERROR) << Name() << " Start exception is:" << e.what();
         return ROBOT_FAILTURE;
    }
    LOG(INFO) << Name() << " Start end.";
    return ROBOT_SUCCESS;
}

void Perception::Stop()
{
   LOG(INFO) << Name() << " Stop start...";
   LOG(INFO) << Name() << " Stop end.";
}


void Perception::CanTopicCallback(const canbus::canMsg::ConstPtr& msg)
{
    unsigned char ucCanMsgSrc[8] = { 0 };
    HexStrToByte(msg->data.c_str(), ucCanMsgSrc, 16);
    CharReverse(ucCanMsgSrc,8);
if(msg->id == robot::common::SysLearnBack)
{
LOG(INFO)<<Name()<<" Receive 6EE";
}
    switch (msg->id) {
    case robot::common::ActuatorEnableStatus:
        PublishState_6E5(ucCanMsgSrc);
        break;
    case robot::common::RSysInfoBack2:
        PublishState_6ED(ucCanMsgSrc);
        break;
    case robot::common::SysLearnBack:
        PublishState_6EE(ucCanMsgSrc);
        break;
    case robot::common::SysWarnErr:
        //SoftStopControl();
        PublishState_6D1(ucCanMsgSrc);
        break;
    case robot::common::AccErr:
        //SoftStopControl();
        PublishState_6D2_3(ucCanMsgSrc, 2, bus_pub_6D2);
        break;
    case robot::common::BrkErr:
        //SoftStopControl();
        PublishState_6D2_3(ucCanMsgSrc, 3, bus_pub_6D3);
        break;
    case robot::common::XsftErr:
        //SoftStopControl();
        PublishState_6D4_6(ucCanMsgSrc, 4, bus_pub_6D4);
        break;
    case robot::common::YsftErr:
        //SoftStopControl();
        PublishState_6D4_6(ucCanMsgSrc, 5, bus_pub_6D5);
        break;
    case robot::common::StrErr:
        //SoftStopControl();
        PublishState_6D4_6(ucCanMsgSrc, 6, bus_pub_6D6);
        break;
    case robot::common::EstopInfo:
        PublishState_2E0(ucCanMsgSrc);
        break;
    case robot::common::EstopErr:
        PublishState_2E1(ucCanMsgSrc);
        break;
    case robot::common::CtrPowerStatus:
        PublishState_6E4(ucCanMsgSrc);
        break;
    case robot::common:: PedalLearnPosBack:
      PublishState_6E1(ucCanMsgSrc);
      break;
    case robot::common::SteerLearnPosBack:
      PublishState_6E2(ucCanMsgSrc);
      break;
    case robot::common::ShiftLearnPosBack1:
      PublishState_6E3(ucCanMsgSrc);
      break;
    case robot::common::ShiftLearnPosBack2:
      PublishState_6EF(ucCanMsgSrc);
      break;
    case robot::common::RSysInfoBack1:
    PublishState_6EB(ucCanMsgSrc);
    break;
    case robot::common::SysMonitorBack:
      PublishState_6D0(ucCanMsgSrc);
      break;
    case robot::common::ZSftStaus:
      PublishState_6F5(ucCanMsgSrc);
      break;
    default:
        break;
    }
}

bool Perception::PublishState_6E5(unsigned char* msg)
{
//    int copySize;
//    if(msg.size() < sizeof(s_state_6E5))
//        copySize = msg->data.size();
//    else
//        copySize = sizeof(s_state_6E5);

    robot::canbusSpace::T_STATE_OUT_6E5 state_6E5;
    memset(&state_6E5, 0 ,sizeof(state_6E5));
    memcpy(&state_6E5, msg, CAN_MSG_LEN);

    if(state_6E5.uiAccClchCtrStat != s_state_6E5.uiAccClchCtrStat || state_6E5.uiAccMotEnStat != s_state_6E5.uiAccMotEnStat ||
            state_6E5.uiBrkMotEnStat != s_state_6E5.uiBrkMotEnStat ||
            state_6E5.uiClchMotEnStat != s_state_6E5.uiClchMotEnStat || state_6E5.uiPauseEnStat != s_state_6E5.uiPauseEnStat ||
            state_6E5.uiStrMotEnStat != s_state_6E5.uiStrMotEnStat || state_6E5.uiXSftMotEnStat != s_state_6E5.uiXSftMotEnStat ||
            state_6E5.uiYSftMotEnStat != s_state_6E5.uiYSftMotEnStat || state_6E5.uiZSftMotEnStat != s_state_6E5.uiZSftMotEnStat)
    {

        memcpy(&s_state_6E5, msg, CAN_MSG_LEN);

        aes_msg.AccClchCtrStat = state_6E5.uiAccClchCtrStat;
        aes_msg.AccMotEnStat =  state_6E5.uiAccMotEnStat;
        aes_msg.BrkMotEnStat = state_6E5.uiBrkMotEnStat;
        aes_msg.ClchMotEnStat = state_6E5.uiClchMotEnStat;
        aes_msg.PauseEnStat = state_6E5.uiPauseEnStat;
        aes_msg.StrMotEnStat = state_6E5.uiStrMotEnStat;
        aes_msg.XSftMotEnStat = state_6E5.uiXSftMotEnStat;
        aes_msg.YSftMotEnStat = state_6E5.uiYSftMotEnStat;
        aes_msg.ZSftMotEnStat = state_6E5.uiZSftMotEnStat;

        return true;
    }
    else
    {
 //       LOG(INFO) << Name() << " PublishState_6E5 data is repeat";
        return false;
    }
}

bool Perception::PublishState_6E4(unsigned char* msg)
{
    robot::canbusSpace::T_STATE_OUT_6E4 state_6E4;
    memset(&state_6E4, 0 ,sizeof(state_6E4));
    memcpy(&state_6E4, msg, CAN_MSG_LEN);
    if(state_6E4.uiAccCtrPowStat != s_state_6E4.uiAccCtrPowStat || state_6E4.uiBrkCtrPowStat != s_state_6E4.uiBrkCtrPowStat ||
            state_6E4.uiClchCtrPowStat != s_state_6E4.uiClchCtrPowStat || state_6E4.uiRCUPowStat != s_state_6E4.uiRCUPowStat ||
            state_6E4.uiStrCtrPowStat != s_state_6E4.uiStrCtrPowStat || state_6E4.uiXSftArmCtrPowStat != s_state_6E4.uiXSftArmCtrPowStat||
            state_6E4.uiYSftArmCtrPowStat != s_state_6E4.uiYSftArmCtrPowStat || state_6E4.uiZSftArmCtrPowStat != s_state_6E4.uiZSftArmCtrPowStat)
    {

        memcpy(&s_state_6E4, msg, CAN_MSG_LEN);
        power_msg.AccCtrPowStat = state_6E4.uiAccCtrPowStat;
        power_msg.BrkCtrPowStat =  state_6E4.uiBrkCtrPowStat;
        power_msg.RCUPowStat = state_6E4.uiRCUPowStat;
        power_msg.StrCtrPowStat = state_6E4.uiStrCtrPowStat;
        power_msg.XSftArmCtrPowStat = state_6E4.uiXSftArmCtrPowStat;
        power_msg.YSftArmCtrPowStat = state_6E4.uiYSftArmCtrPowStat;
        power_msg.ClchCtrPowStat = state_6E4.uiClchCtrPowStat;
        power_msg.ZSftArmCtrPowStat = state_6E4.uiZSftArmCtrPowStat;

        return true;
    }
    else
    {
 //       LOG(INFO) << Name() << " PublishState_6E4 data is repeat";
        return false;
    }
}

bool Perception::PublishState_6EE(unsigned char* msg)
{


//    int copySize;
//    if(msg->data.size() < sizeof(s_state_6EE))
//        copySize = msg->data.size();
//    else
//        copySize = sizeof(s_state_6EE);

    robot::canbusSpace::T_STATE_OUT_6EE state_6EE;
    memset(&state_6EE, 0 ,sizeof(state_6EE));
    memcpy(&state_6EE,  msg, CAN_MSG_LEN);

    if(state_6EE.uiAcc0LrnStat != s_state_6EE.uiAcc0LrnStat || state_6EE.uiAccLrnStat != s_state_6EE.uiAccLrnStat ||
            state_6EE.uiAccMaxLrnStat != s_state_6EE.uiAccMaxLrnStat || state_6EE.uiBrk0LrnStat != s_state_6EE.uiBrk0LrnStat ||
            state_6EE.uiBrkLrnStat != s_state_6EE.uiBrkLrnStat || state_6EE.uiBrkMaxLrnStat != s_state_6EE.uiBrkMaxLrnStat ||
            state_6EE.uiClch0LrnStat != s_state_6EE.uiClch0LrnStat || state_6EE.uiClchLrnStat != s_state_6EE.uiClchLrnStat ||
            state_6EE.uiClchMaxLrnStat != s_state_6EE.uiClchMaxLrnStat || state_6EE.uiDLrnStat != s_state_6EE.uiDLrnStat ||
            state_6EE.uiDpLrnStat != s_state_6EE.uiDpLrnStat || state_6EE.uiMLrnStat != s_state_6EE.uiMLrnStat ||
            state_6EE.uiNLrnStat != s_state_6EE.uiNLrnStat || state_6EE.uiPLrnStat != s_state_6EE.uiPLrnStat ||
            state_6EE.uiRLrnStat != s_state_6EE.uiRLrnStat || state_6EE.uiSftLrnStat != s_state_6EE.uiSftLrnStat ||
            state_6EE.uiStr0LrnStat != s_state_6EE.uiStr0LrnStat || state_6EE.uiStrLLrnStat != s_state_6EE.uiStrLLrnStat ||
            state_6EE.uiStrLrnStat != s_state_6EE.uiStrLrnStat || state_6EE.uiStrRLrnStat != s_state_6EE.uiStrRLrnStat ||
            state_6EE.uiUpLrnStat != s_state_6EE.uiUpLrnStat || state_6EE.uiAccLrnWay != s_state_6EE.uiAccLrnWay ||
            state_6EE.uiBrkLrnWay != s_state_6EE.uiBrkLrnWay || state_6EE.uiStrLrnWay != s_state_6EE.uiStrLrnWay ||
            state_6EE.uiClutchLrnWay != s_state_6EE.uiClutchLrnWay)
    {

        memcpy(&s_state_6EE,  msg, CAN_MSG_LEN);

        sl_msg.Acc0LrnStat = state_6EE.uiAcc0LrnStat;
        sl_msg.AccLrnStat = state_6EE.uiAccLrnStat;
        sl_msg.AccMaxLrnStat = state_6EE.uiAccMaxLrnStat;
        sl_msg.Brk0LrnStat = state_6EE.uiBrk0LrnStat;
        sl_msg.BrkLrnStat = state_6EE.uiBrkLrnStat;
        sl_msg.BrkMaxLrnStat = state_6EE.uiBrkMaxLrnStat;
        sl_msg.Clch0LrnStat = state_6EE.uiClch0LrnStat;
        sl_msg.ClchLrnStat = state_6EE.uiClchLrnStat;
        sl_msg.ClchMaxLrnStat = state_6EE.uiClchMaxLrnStat;
        sl_msg.DLrnStat = state_6EE.uiDLrnStat;
        sl_msg.DpLrnStat = state_6EE.uiDpLrnStat;
        sl_msg.MLrnStat = state_6EE.uiMLrnStat;
        sl_msg.NLrnStat = state_6EE.uiNLrnStat;
        sl_msg.PLrnStat = state_6EE.uiPLrnStat;
        sl_msg.RLrnStat = state_6EE.uiRLrnStat;
        sl_msg.SftLrnStat = state_6EE.uiSftLrnStat;
        sl_msg.Str0LrnStat = state_6EE.uiStr0LrnStat;
        sl_msg.StrLLrnStat = state_6EE.uiStrLLrnStat;
        sl_msg.StrLrnStat = state_6EE.uiStrLrnStat;
        sl_msg.StrRLrnStat = state_6EE.uiStrRLrnStat;
        sl_msg.UpLrnStat = state_6EE.uiUpLrnStat;
        sl_msg.AccLrnWay = state_6EE.uiAccLrnWay;
        sl_msg.BrkLrnWay = state_6EE.uiBrkLrnWay;
        sl_msg.StrLrnWay = state_6EE.uiStrLrnWay;
        sl_msg.ClutchLrnWay = state_6EE.uiClutchLrnWay;

        return true;
    }
    else
    {
   //     LOG(INFO) << Name() << " PublishState_6EE data is repeat";
        return false;
    }
}

bool Perception::PublishState_6ED(unsigned char* msg)
{


//    int copySize;
//    if(msg->data.size() < sizeof(s_state_6ED))
//        copySize = msg->data.size();
//    else
//        copySize = sizeof(s_state_6ED);

    robot::canbusSpace::T_STATE_OUT_6ED state_6ED;
    memset(&state_6ED, 0 ,sizeof(state_6ED));
    memcpy(&state_6ED,  msg, CAN_MSG_LEN);

    if(state_6ED.uiAccCtrAlive != s_state_6ED.uiAccCtrAlive || state_6ED.uiAccMotClbr != s_state_6ED.uiAccMotClbr ||
            state_6ED.uiBrkCtrAlive != s_state_6ED.uiBrkCtrAlive || state_6ED.uiBrkMotClbr != s_state_6ED.uiBrkMotClbr ||
            state_6ED.uiClchCtrAlive != s_state_6ED.uiClchCtrAlive || state_6ED.uiClchMotClbr != s_state_6ED.uiClchMotClbr ||
            state_6ED.uiCtrMdStat != s_state_6ED.uiCtrMdStat || state_6ED.uiRCUAlive != s_state_6ED.uiRCUAlive ||
            state_6ED.uiStrCtrAlive != s_state_6ED.uiStrCtrAlive || state_6ED.uiStrMotClbr != s_state_6ED.uiStrMotClbr ||
            state_6ED.uiXSftArmCtrAlive != s_state_6ED.uiXSftArmCtrAlive || state_6ED.uiXSftMotClbr != s_state_6ED.uiXSftMotClbr ||
            state_6ED.uiYSftArmCtrAlive != s_state_6ED.uiYSftArmCtrAlive || state_6ED.uiYSftMotClbr != s_state_6ED.uiYSftMotClbr)
    {

        memcpy(&s_state_6ED,  msg, CAN_MSG_LEN);

        si2_msg.AccCtrAlive = state_6ED.uiAccCtrAlive;
        si2_msg.AccMotClbr = state_6ED.uiAccMotClbr;
        si2_msg.BrkCtrAlive = state_6ED.uiBrkCtrAlive;
        si2_msg.BrkMotClbr = state_6ED.uiBrkMotClbr;
        si2_msg.ClchCtrAlive = state_6ED.uiClchCtrAlive;
        si2_msg.ClchMotClbr = state_6ED.uiClchMotClbr;
        si2_msg.CtrMdStat = state_6ED.uiCtrMdStat;
        si2_msg.RCUAlive = state_6ED.uiRCUAlive;
        si2_msg.StrCtrAlive = state_6ED.uiStrCtrAlive;
        si2_msg.StrMotClbr = state_6ED.uiStrMotClbr;
        si2_msg.XSftArmCtrAlive = state_6ED.uiXSftArmCtrAlive;
        si2_msg.XSftMotClbr = state_6ED.uiXSftMotClbr;
        si2_msg.YSftArmCtrAlive = state_6ED.uiYSftArmCtrAlive;
        si2_msg.YSftMotClbr = state_6ED.uiYSftMotClbr;
        si2_msg.RbtCfgMdStat = state_6ED.uiRbtCfgMdStat;

        return true;
    }
    else
    {
  //      LOG(INFO) << Name() << " PublishState_6ED data is repeat";
        return false;
    }
}

bool Perception::PublishState_6D1(unsigned char* msg)
{


//    int copySize;
//    if(msg->data.size() < sizeof(s_state_6D1))
//        copySize = msg->data.size();
//    else
//        copySize = sizeof(s_state_6D1);

    robot::canbusSpace::T_STATE_OUT_6D1 state_6D1;
    memset(&state_6D1, 0 ,sizeof(state_6D1));
    memcpy(&state_6D1, msg, CAN_MSG_LEN);

    if(state_6D1.uiSysErra != s_state_6D1.uiSysErra || state_6D1.uiSysErrb != s_state_6D1.uiSysErrb || state_6D1.uiSysErrc != s_state_6D1.uiSysErrc ||
            state_6D1.uiSysErrd != s_state_6D1.uiSysErrd || state_6D1.uiSysErre != s_state_6D1.uiSysErre || state_6D1.uiSysErrf != s_state_6D1.uiSysErrf ||
            state_6D1.uiSysErrg != s_state_6D1.uiSysErrg || state_6D1.uiSysErrh != s_state_6D1.uiSysErrh || state_6D1.uiSysErri != s_state_6D1.uiSysErri ||
            state_6D1.uiSysErrj != s_state_6D1.uiSysErrj || state_6D1.uiSysErrk != s_state_6D1.uiSysErrk || state_6D1.uiSysErrl != s_state_6D1.uiSysErrl ||
            state_6D1.uiSysErrm != s_state_6D1.uiSysErrm || state_6D1.uiSysErrn != s_state_6D1.uiSysErrn || state_6D1.uiSysErro != s_state_6D1.uiSysErro ||
            state_6D1.uiSysErrp != s_state_6D1.uiSysErrp || state_6D1.uiSysErrq != s_state_6D1.uiSysErrq || state_6D1.uiSysErrr != s_state_6D1.uiSysErrr ||
            state_6D1.uiSysErrs != s_state_6D1.uiSysErrs || state_6D1.uiSysErrt != s_state_6D1.uiSysErrt)
    {

        memcpy(&s_state_6D1, msg, CAN_MSG_LEN);

        se_msg.SysErra = state_6D1.uiSysErra;
        se_msg.SysErrb = state_6D1.uiSysErrb;
        se_msg.SysErrc = state_6D1.uiSysErrc;
        se_msg.SysErrd = state_6D1.uiSysErrd;
        se_msg.SysErre = state_6D1.uiSysErre;
        se_msg.SysErrf = state_6D1.uiSysErrf;
        se_msg.SysErrg = state_6D1.uiSysErrg;
        se_msg.SysErrh = state_6D1.uiSysErrh;
        se_msg.SysErri = state_6D1.uiSysErri;
        se_msg.SysErrj = state_6D1.uiSysErrj;
        se_msg.SysErrk = state_6D1.uiSysErrk;
        se_msg.SysErrl = state_6D1.uiSysErrl;
        se_msg.SysErrm = state_6D1.uiSysErrm;
        se_msg.SysErrn = state_6D1.uiSysErrn;
        se_msg.SysErro = state_6D1.uiSysErro;
        se_msg.SysErrp = state_6D1.uiSysErrp;
        se_msg.SysErrq = state_6D1.uiSysErrq;
        se_msg.SysErrr = state_6D1.uiSysErrr;
        se_msg.SysErrs = state_6D1.uiSysErrs;
        se_msg.SysErrt = state_6D1.uiSysErrt;


        return true;
    }
    else
    {
   //     LOG(INFO) << Name() << " PublishState_6D1 data is repeat";
        return false;
    }
}

bool Perception::PublishState_6D2_3(unsigned char* msg, int type, const ros::Publisher &pub)
{


    robot::canbusSpace::T_MOT_CTR_ERR_1 *pErr;
    switch (type) {
    case 2:
        pErr = &s_state_6D2;
        break;
    case 3:
        pErr = &s_state_6D3;
        break;
    default:
        LOG(INFO) << Name() << " PublishState_6D2_3 " << type << " type is unvalid";
        return false;
        break;
    }

//    int copySize;
//    if(msg->data.size() < sizeof(s_state_6D2))
//        copySize = msg->data.size();
//    else
//        copySize = sizeof(s_state_6D2);

    //6D2_2 6D2_3 数据结构相同
    robot::canbusSpace::T_MOT_CTR_ERR_1 state_6D2;
    memset(&state_6D2, 0 ,sizeof(state_6D2));
    memcpy(&state_6D2, msg, CAN_MSG_LEN);

    if(state_6D2.a != pErr->a || state_6D2.aa != pErr->aa || state_6D2.ab != pErr->ab || state_6D2.ac != pErr->ac
       || state_6D2.ad != pErr->ad || state_6D2.ae != pErr->ae || state_6D2.af != pErr->af || state_6D2.ag != pErr->ag
       || state_6D2.ah != pErr->ah || state_6D2.ai != pErr->ai || state_6D2.aj != pErr->aj || state_6D2.ak != pErr->ak
       || state_6D2.al != pErr->al || state_6D2.am != pErr->am || state_6D2.an != pErr->an || state_6D2.ao != pErr->ao
       || state_6D2.ap != pErr->ap || state_6D2.aq != pErr->aq || state_6D2.ar != pErr->ar || state_6D2.as != pErr->as
       || state_6D2.at != pErr->at || state_6D2.au != pErr->au || state_6D2.av != pErr->av || state_6D2.aw != pErr->aw
       || state_6D2.ax != pErr->ax || state_6D2.ay != pErr->ay || state_6D2.az != pErr->az || state_6D2.b != pErr->b
       || state_6D2.ba != pErr->ba || state_6D2.bb != pErr->bb || state_6D2.bc != pErr->bc || state_6D2.bd != pErr->be
       || state_6D2.bf != pErr->bf || state_6D2.bg != pErr->bg || state_6D2.bh != pErr->bh || state_6D2.bi != pErr->bi
       || state_6D2.bj != pErr->bj || state_6D2.bk != pErr->bk || state_6D2.bl != pErr->bl || state_6D2.c != pErr->c
       || state_6D2.d != pErr->d || state_6D2.e != pErr->e || state_6D2.f != pErr->f || state_6D2.g != pErr->g
       || state_6D2.h != pErr->h || state_6D2.i != pErr->i || state_6D2.j != pErr->j || state_6D2.k != pErr->k
       || state_6D2.l != pErr->l || state_6D2.m != pErr->m || state_6D2.n != pErr->n || state_6D2.o != pErr->o
       || state_6D2.p != pErr->p || state_6D2.q != pErr->q || state_6D2.r != pErr->r || state_6D2.s != pErr->s
       || state_6D2.t != pErr->t || state_6D2.u != pErr->u || state_6D2.v != pErr->v || state_6D2.w != pErr->w
       || state_6D2.x != pErr->x || state_6D2.y != pErr->y || state_6D2.z != pErr->z)
    {


        switch (type) {
        case 2:
             memcpy(&s_state_6D2, msg, CAN_MSG_LEN);
            break;
        case 3:
             memcpy(&s_state_6D3, msg, CAN_MSG_LEN);
            break;
        default:
            LOG(INFO) << Name() << " PublishState_6D2_3 " << type << " type is unvalid";
            return false;
            break;
        }

        if(type == 2)
        {
            cec_msg22.Erra = state_6D2.a;
            cec_msg22.Erraa = state_6D2.aa;
            cec_msg22.Errab = state_6D2.ab;
            cec_msg22.Errac = state_6D2.ac;
            cec_msg22.Errad = state_6D2.ad;
            cec_msg22.Errae = state_6D2.ae;
            cec_msg22.Erraf = state_6D2.af;
            cec_msg22.Errag = state_6D2.ag;
            cec_msg22.Errah = state_6D2.ah;
            cec_msg22.Errai = state_6D2.ai;
            cec_msg22.Erraj = state_6D2.aj;
            cec_msg22.Errak = state_6D2.ak;
            cec_msg22.Erral = state_6D2.al;
            cec_msg22.Erram = state_6D2.am;
            cec_msg22.Erran = state_6D2.an;
            cec_msg22.Errao = state_6D2.ao;
            cec_msg22.Errap = state_6D2.ap;
            cec_msg22.Erraq = state_6D2.aq;
            cec_msg22.Errar = state_6D2.ar;
            cec_msg22.Erras = state_6D2.as;
            cec_msg22.Errat = state_6D2.at;
            cec_msg22.Errau = state_6D2.au;
            cec_msg22.Errav = state_6D2.av;
            cec_msg22.Erraw = state_6D2.aw;
            cec_msg22.Errax = state_6D2.ax;
            cec_msg22.Erray = state_6D2.ay;
            cec_msg22.Erraz = state_6D2.az;
            cec_msg22.Errb = state_6D2.b;
            cec_msg22.Errba = state_6D2.ba;
            cec_msg22.Errbb = state_6D2.bb;
            cec_msg22.Errbc = state_6D2.bc;
            cec_msg22.Errbd = state_6D2.bd;
            cec_msg22.Errbe = state_6D2.be;
            cec_msg22.Errbf = state_6D2.bf;
            cec_msg22.Errbg = state_6D2.bg;
            cec_msg22.Errbh = state_6D2.bh;
            cec_msg22.Errbi = state_6D2.bi;
            cec_msg22.Errbk = state_6D2.bk;
            cec_msg22.Errbj = state_6D2.bj;
            cec_msg22.Errbl = state_6D2.bl;
            cec_msg22.Errc = state_6D2.c;
            cec_msg22.Errd = state_6D2.d;
            cec_msg22.Erre = state_6D2.e;
            cec_msg22.Errf = state_6D2.f;
            cec_msg22.Errg = state_6D2.g;
            cec_msg22.Errh = state_6D2.h;
            cec_msg22.Erri = state_6D2.i;
            cec_msg22.Errj = state_6D2.j;
            cec_msg22.Errk = state_6D2.k;
            cec_msg22.Errl = state_6D2.l;
            cec_msg22.Errm = state_6D2.m;
            cec_msg22.Errn = state_6D2.n;
            cec_msg22.Erro = state_6D2.o;
            cec_msg22.Errp = state_6D2.p;
            cec_msg22.Errq = state_6D2.q;
            cec_msg22.Errr = state_6D2.r;
            cec_msg22.Errs = state_6D2.s;
            cec_msg22.Errt = state_6D2.t;
            cec_msg22.Erru = state_6D2.u;
            cec_msg22.Errv = state_6D2.v;
            cec_msg22.Errw = state_6D2.w;
            cec_msg22.Errx = state_6D2.x;
            cec_msg22.Erry = state_6D2.y;
            cec_msg22.Errz = state_6D2.z;
        }
        else if(type == 3)
        {
          cec_msg23.Erra = state_6D2.a;
          cec_msg23.Erraa = state_6D2.aa;
          cec_msg23.Errab = state_6D2.ab;
          cec_msg23.Errac = state_6D2.ac;
          cec_msg23.Errad = state_6D2.ad;
          cec_msg23.Errae = state_6D2.ae;
          cec_msg23.Erraf = state_6D2.af;
          cec_msg23.Errag = state_6D2.ag;
          cec_msg23.Errah = state_6D2.ah;
          cec_msg23.Errai = state_6D2.ai;
          cec_msg23.Erraj = state_6D2.aj;
          cec_msg23.Errak = state_6D2.ak;
          cec_msg23.Erral = state_6D2.al;
          cec_msg23.Erram = state_6D2.am;
          cec_msg23.Erran = state_6D2.an;
          cec_msg23.Errao = state_6D2.ao;
          cec_msg23.Errap = state_6D2.ap;
          cec_msg23.Erraq = state_6D2.aq;
          cec_msg23.Errar = state_6D2.ar;
          cec_msg23.Erras = state_6D2.as;
          cec_msg23.Errat = state_6D2.at;
          cec_msg23.Errau = state_6D2.au;
          cec_msg23.Errav = state_6D2.av;
          cec_msg23.Erraw = state_6D2.aw;
          cec_msg23.Errax = state_6D2.ax;
          cec_msg23.Erray = state_6D2.ay;
          cec_msg23.Erraz = state_6D2.az;
          cec_msg23.Errb = state_6D2.b;
          cec_msg23.Errba = state_6D2.ba;
          cec_msg23.Errbb = state_6D2.bb;
          cec_msg23.Errbc = state_6D2.bc;
          cec_msg23.Errbd = state_6D2.bd;
          cec_msg23.Errbe = state_6D2.be;
          cec_msg23.Errbf = state_6D2.bf;
          cec_msg23.Errbg = state_6D2.bg;
          cec_msg23.Errbh = state_6D2.bh;
          cec_msg23.Errbi = state_6D2.bi;
          cec_msg23.Errbk = state_6D2.bk;
          cec_msg23.Errbj = state_6D2.bj;
          cec_msg23.Errbl = state_6D2.bl;
          cec_msg23.Errc = state_6D2.c;
          cec_msg23.Errd = state_6D2.d;
          cec_msg23.Erre = state_6D2.e;
          cec_msg23.Errf = state_6D2.f;
          cec_msg23.Errg = state_6D2.g;
          cec_msg23.Errh = state_6D2.h;
          cec_msg23.Erri = state_6D2.i;
          cec_msg23.Errj = state_6D2.j;
          cec_msg23.Errk = state_6D2.k;
          cec_msg23.Errl = state_6D2.l;
          cec_msg23.Errm = state_6D2.m;
          cec_msg23.Errn = state_6D2.n;
          cec_msg23.Erro = state_6D2.o;
          cec_msg23.Errp = state_6D2.p;
          cec_msg23.Errq = state_6D2.q;
          cec_msg23.Errr = state_6D2.r;
          cec_msg23.Errs = state_6D2.s;
          cec_msg23.Errt = state_6D2.t;
          cec_msg23.Erru = state_6D2.u;
          cec_msg23.Errv = state_6D2.v;
          cec_msg23.Errw = state_6D2.w;
          cec_msg23.Errx = state_6D2.x;
          cec_msg23.Erry = state_6D2.y;
          cec_msg23.Errz = state_6D2.z;
        }

        return true;
    }
    else
    {
    //    LOG(INFO) << Name() << " PublishState_6D2_3 " << type << " data is repeat";
        return false;
    }
}

bool Perception::PublishState_6D4_6(unsigned char* msg, int type, const ros::Publisher &pub)
{


    robot::canbusSpace::T_ACC_ERR *pErr;
    switch (type) {
    case 4:
        pErr = (robot::canbusSpace::T_ACC_ERR *)&s_state_6D4;
        break;
    case 5:
        pErr = (robot::canbusSpace::T_ACC_ERR *)&s_state_6D5;
        break;
    case 6:
        pErr = (robot::canbusSpace::T_ACC_ERR *)&s_state_6D6;
        break;
    default:
        LOG(INFO) << Name() << " PublishState_6D4_6 " << type << " type is unvalid";
        return false;
        break;
    }

//    int copySize;
//    if(msg->data.size() < sizeof(s_state_6D2))
//        copySize = msg->data.size();
//    else
//        copySize = sizeof(s_state_6D2);

    // 6D2_4 6D2_5 6D2_6 数据结构相同
    robot::canbusSpace::T_ACC_ERR state_6D4;
    memset(&state_6D4, 0 ,sizeof(state_6D4));
    memcpy(&state_6D4, msg, CAN_MSG_LEN);

    if(state_6D4.uiAccErra != pErr->uiAccErra || state_6D4.uiAccErraa != pErr->uiAccErraa || state_6D4.uiAccErrab != pErr->uiAccErrab ||
            state_6D4.uiAccErrac != pErr->uiAccErrac || state_6D4.uiAccErrad != pErr->uiAccErrad || state_6D4.uiAccErrae != pErr->uiAccErrae ||
            state_6D4.uiAccErraf != pErr->uiAccErraf || state_6D4.uiAccErrag != pErr->uiAccErrag || state_6D4.uiAccErrah != pErr->uiAccErrah ||
            state_6D4.uiAccErrai != pErr->uiAccErrai || state_6D4.uiAccErrb != pErr->uiAccErrb || state_6D4.uiAccErrc != pErr->uiAccErrc ||
            state_6D4.uiAccErrd != pErr->uiAccErrd || state_6D4.uiAccErre != pErr->uiAccErre || state_6D4.uiAccErrf != pErr->uiAccErrf ||
            state_6D4.uiAccErrg != pErr->uiAccErrg || state_6D4.uiAccErrh != pErr->uiAccErrh || state_6D4.uiAccErrai != pErr->uiAccErrai ||
            state_6D4.uiAccErrj != pErr->uiAccErrj || state_6D4.uiAccErrk != pErr->uiAccErrk || state_6D4.uiAccErrl != pErr->uiAccErrl ||
            state_6D4.uiAccErrm != pErr->uiAccErrm || state_6D4.uiAccErrn != pErr->uiAccErrn || state_6D4.uiAccErro != pErr->uiAccErro ||
            state_6D4.uiAccErrp != pErr->uiAccErrp || state_6D4.uiAccErrq != pErr->uiAccErrq || state_6D4.uiAccErrr != pErr->uiAccErrr ||
            state_6D4.uiAccErrs != pErr->uiAccErrs || state_6D4.uiAccErrt != pErr->uiAccErrt || state_6D4.uiAccErru != pErr->uiAccErru ||
            state_6D4.uiAccErrv != pErr->uiAccErrv || state_6D4.uiAccErrw != pErr->uiAccErrw || state_6D4.uiAccErrx != pErr->uiAccErrx ||
            state_6D4.uiAccErry != pErr->uiAccErry || state_6D4.uiAccErrz != pErr->uiAccErrz)
    {


        switch (type) {
        case 4:
             memcpy(&s_state_6D4, msg, CAN_MSG_LEN);
            break;
        case 5:
             memcpy(&s_state_6D5, msg, CAN_MSG_LEN);
            break;
        case 6:
             memcpy(&s_state_6D6, msg, CAN_MSG_LEN);
            break;
        default:
            LOG(INFO) << Name() << " PublishState_6D4_6 " << type << " type is unvalid";
            return false;
            break;
        }

        if(type ==4)
        {
          cec_msg4.Erra = state_6D4.uiAccErra;
          cec_msg4.Erraa = state_6D4.uiAccErraa;
          cec_msg4.Errab = state_6D4.uiAccErrab;
          cec_msg4.Errac = state_6D4.uiAccErrac;
          cec_msg4.Errad = state_6D4.uiAccErrad;
          cec_msg4.Errae = state_6D4.uiAccErrae;
          cec_msg4.Erraf = state_6D4.uiAccErraf;
          cec_msg4.Errag = state_6D4.uiAccErrag;
          cec_msg4.Errah = state_6D4.uiAccErrah;
          cec_msg4.Errai = state_6D4.uiAccErrai;
          cec_msg4.Errb = state_6D4.uiAccErrb;
          cec_msg4.Errc = state_6D4.uiAccErrc;
          cec_msg4.Errd = state_6D4.uiAccErrd;
          cec_msg4.Erre = state_6D4.uiAccErre;
          cec_msg4.Errf = state_6D4.uiAccErrf;
          cec_msg4.Errg = state_6D4.uiAccErrg;
          cec_msg4.Errh = state_6D4.uiAccErrh;
          cec_msg4.Erri = state_6D4.uiAccErri;
          cec_msg4.Errj = state_6D4.uiAccErrj;
          cec_msg4.Errk = state_6D4.uiAccErrk;
          cec_msg4.Errl = state_6D4.uiAccErrl;
          cec_msg4.Errm = state_6D4.uiAccErrm;
          cec_msg4.Errn = state_6D4.uiAccErrn;
          cec_msg4.Erro = state_6D4.uiAccErro;
          cec_msg4.Errp = state_6D4.uiAccErrp;
          cec_msg4.Errq = state_6D4.uiAccErrq;
          cec_msg4.Errr = state_6D4.uiAccErrr;
          cec_msg4.Errs = state_6D4.uiAccErrs;
          cec_msg4.Errt = state_6D4.uiAccErrt;
          cec_msg4.Erru = state_6D4.uiAccErru;
          cec_msg4.Errv = state_6D4.uiAccErrv;
          cec_msg4.Errw = state_6D4.uiAccErrw;
          cec_msg4.Errx = state_6D4.uiAccErrx;
          cec_msg4.Erry = state_6D4.uiAccErry;
          cec_msg4.Errz = state_6D4.uiAccErrz;
        }
        else if(type == 5)
        {
          cec_msg5.Erra = state_6D4.uiAccErra;
          cec_msg5.Erraa = state_6D4.uiAccErraa;
          cec_msg5.Errab = state_6D4.uiAccErrab;
          cec_msg5.Errac = state_6D4.uiAccErrac;
          cec_msg5.Errad = state_6D4.uiAccErrad;
          cec_msg5.Errae = state_6D4.uiAccErrae;
          cec_msg5.Erraf = state_6D4.uiAccErraf;
          cec_msg5.Errag = state_6D4.uiAccErrag;
          cec_msg5.Errah = state_6D4.uiAccErrah;
          cec_msg5.Errai = state_6D4.uiAccErrai;
          cec_msg5.Errb = state_6D4.uiAccErrb;
          cec_msg5.Errc = state_6D4.uiAccErrc;
          cec_msg5.Errd = state_6D4.uiAccErrd;
          cec_msg5.Erre = state_6D4.uiAccErre;
          cec_msg5.Errf = state_6D4.uiAccErrf;
          cec_msg5.Errg = state_6D4.uiAccErrg;
          cec_msg5.Errh = state_6D4.uiAccErrh;
          cec_msg5.Erri = state_6D4.uiAccErri;
          cec_msg5.Errj = state_6D4.uiAccErrj;
          cec_msg5.Errk = state_6D4.uiAccErrk;
          cec_msg5.Errl = state_6D4.uiAccErrl;
          cec_msg5.Errm = state_6D4.uiAccErrm;
          cec_msg5.Errn = state_6D4.uiAccErrn;
          cec_msg5.Erro = state_6D4.uiAccErro;
          cec_msg5.Errp = state_6D4.uiAccErrp;
          cec_msg5.Errq = state_6D4.uiAccErrq;
          cec_msg5.Errr = state_6D4.uiAccErrr;
          cec_msg5.Errs = state_6D4.uiAccErrs;
          cec_msg5.Errt = state_6D4.uiAccErrt;
          cec_msg5.Erru = state_6D4.uiAccErru;
          cec_msg5.Errv = state_6D4.uiAccErrv;
          cec_msg5.Errw = state_6D4.uiAccErrw;
          cec_msg5.Errx = state_6D4.uiAccErrx;
          cec_msg5.Erry = state_6D4.uiAccErry;
          cec_msg5.Errz = state_6D4.uiAccErrz;
        }
        else if(type == 6)
        {
          cec_msg6.Erra = state_6D4.uiAccErra;
          cec_msg6.Erraa = state_6D4.uiAccErraa;
          cec_msg6.Errab = state_6D4.uiAccErrab;
          cec_msg6.Errac = state_6D4.uiAccErrac;
          cec_msg6.Errad = state_6D4.uiAccErrad;
          cec_msg6.Errae = state_6D4.uiAccErrae;
          cec_msg6.Erraf = state_6D4.uiAccErraf;
          cec_msg6.Errag = state_6D4.uiAccErrag;
          cec_msg6.Errah = state_6D4.uiAccErrah;
          cec_msg6.Errai = state_6D4.uiAccErrai;
          cec_msg6.Errb = state_6D4.uiAccErrb;
          cec_msg6.Errc = state_6D4.uiAccErrc;
          cec_msg6.Errd = state_6D4.uiAccErrd;
          cec_msg6.Erre = state_6D4.uiAccErre;
          cec_msg6.Errf = state_6D4.uiAccErrf;
          cec_msg6.Errg = state_6D4.uiAccErrg;
          cec_msg6.Errh = state_6D4.uiAccErrh;
          cec_msg6.Erri = state_6D4.uiAccErri;
          cec_msg6.Errj = state_6D4.uiAccErrj;
          cec_msg6.Errk = state_6D4.uiAccErrk;
          cec_msg6.Errl = state_6D4.uiAccErrl;
          cec_msg6.Errm = state_6D4.uiAccErrm;
          cec_msg6.Errn = state_6D4.uiAccErrn;
          cec_msg6.Erro = state_6D4.uiAccErro;
          cec_msg6.Errp = state_6D4.uiAccErrp;
          cec_msg6.Errq = state_6D4.uiAccErrq;
          cec_msg6.Errr = state_6D4.uiAccErrr;
          cec_msg6.Errs = state_6D4.uiAccErrs;
          cec_msg6.Errt = state_6D4.uiAccErrt;
          cec_msg6.Erru = state_6D4.uiAccErru;
          cec_msg6.Errv = state_6D4.uiAccErrv;
          cec_msg6.Errw = state_6D4.uiAccErrw;
          cec_msg6.Errx = state_6D4.uiAccErrx;
          cec_msg6.Erry = state_6D4.uiAccErry;
          cec_msg6.Errz = state_6D4.uiAccErrz;
        }

        return true;
    }
    else
    {
    //    LOG(INFO) << Name() << " PublishState_6D4_6 " << type << " data is repeat";
        return false;
    }
}

bool Perception::PublishState_2E0(unsigned char* msg)
{
//    int copySize;
//    if(msg->data.size() < sizeof(s_state_2E0))
//    {
//        copySize = msg->data.size();
//    }
//    else
//        copySize = sizeof(s_state_2E0);

    robot::canbusSpace::T_ESTOP_2E0 state_2E0;
    memset(&state_2E0, 0 ,sizeof(state_2E0));
    memcpy(&state_2E0, msg, CAN_MSG_LEN);

    if(state_2E0.uiEstpAbnStat != s_state_2E0.uiEstpAbnStat || state_2E0.uiEstpAlive != state_2E0.uiEstpAlive ||
            state_2E0.uiEstpEnStat != s_state_2E0.uiEstpEnStat || state_2E0.uiPmpEnStat != s_state_2E0.uiPmpEnStat ||
       state_2E0.uiPmpPressureStat != s_state_2E0.uiPmpPressureStat || state_2E0.uiPmpValveStat != s_state_2E0.uiPmpValveStat)
    {

        memcpy(&s_state_2E0,msg, CAN_MSG_LEN);

        ei_msg.EstpAbnStat = state_2E0.uiEstpAbnStat;
        ei_msg.EstpAlive = state_2E0.uiEstpAlive;
        ei_msg.EstpEnStat = state_2E0.uiEstpEnStat;
        ei_msg.PmpEnStat = state_2E0.uiPmpEnStat;

        return true;
    }
    else
    {
    //    LOG(INFO) << Name() << " PublishState_2E0 data is repeat";
        return false;
    }
}

bool Perception::PublishState_2E1(unsigned char* msg)
{
//    int copySize;
//    if(msg->data.size() < sizeof(s_state_2E1))
//        copySize = msg->data.size();
//    else
//        copySize = sizeof(s_state_2E1);

    robot::canbusSpace::T_ESTOP_2E1 state_2E1;
    memset(&state_2E1, 0 ,sizeof(state_2E1));
    memcpy(&state_2E1, msg, CAN_MSG_LEN);

    if(state_2E1.uiEspErra != s_state_2E1.uiEspErra || state_2E1.uiEspErrb != s_state_2E1.uiEspErrb || state_2E1.uiEspErrc != s_state_2E1.uiEspErrc ||
            state_2E1.uiEspErrd != s_state_2E1.uiEspErrd || state_2E1.uiEspErre != s_state_2E1.uiEspErre || state_2E1.uiEspErrf != s_state_2E1.uiEspErrf)
    {

        memcpy(&s_state_2E1,  msg, CAN_MSG_LEN);

        ee_msg.EspErra = state_2E1.uiEspErra;
        ee_msg.EspErrb = state_2E1.uiEspErrb;
        ee_msg.EspErrc = state_2E1.uiEspErrc;
        ee_msg.EspErrd = state_2E1.uiEspErrd;
        ee_msg.EspErre = state_2E1.uiEspErre;
        ee_msg.EspErrf = state_2E1.uiEspErrf;

        return true;
    }
    else
    {
    //    LOG(INFO) << Name() << " PublishState_2E1 data is repeat";
        return false;
    }
}

bool Perception::SoftStopControl()
{
    canbus::EstpCtrl srvEstp;
    canbus::SysCtrlMode srvSysMode;
    // 系统软急停使能: enable
    srvEstp.request.uiPauseEnReq = 1;
    // 系统控制模式请求: 人工驾驶
    srvSysMode.request.uiCtrMdReq = 3;

    if (clientActuatorEnableRequest.call(srvEstp) && clientActuatorEnableRequest.call(srvSysMode))
    {
        LOG(INFO) << Name() << " SoftStopControl call service return success, value = " << srvEstp.response.uiReturn;
        return true;
    }
    else
    {
        LOG(ERROR) << Name() << " SoftStopControl call service return fail";
        return false;
    }
}

bool Perception::PublishState_6E1(unsigned char* msg)
{
  robot::canbusSpace::T_STATE_OUT_6E1 state_6E1;
  memset(&state_6E1, 0 ,sizeof(state_6E1));
  memcpy(&state_6E1, msg, CAN_MSG_LEN);
  if(state_6E1.uiClbtAccPsng0 != s_state_6E1.uiClbtAccPsng0 || state_6E1.uiClbtAccPsngMax != s_state_6E1.uiClbtAccPsngMax ||
          state_6E1.uiClbtBrkPsng0 != s_state_6E1.uiClbtBrkPsng0 || state_6E1.uiClbtBrkPsngMax != s_state_6E1.uiClbtBrkPsngMax)
  {

      memcpy(&s_state_6E1, msg, CAN_MSG_LEN);

      pedLrnPos_msg.ClbtAccPsng0 = state_6E1.uiClbtAccPsng0;
      pedLrnPos_msg.ClbtAccPsngMax =  state_6E1.uiClbtAccPsngMax;
      pedLrnPos_msg.ClbtBrkPsng0 = state_6E1.uiClbtBrkPsng0;
      pedLrnPos_msg.ClbtBrkPsngMax = state_6E1.uiClbtBrkPsngMax;

      return true;
  }
  else
  {
   //   LOG(INFO) << Name() << " PublishState_6E1 data is repeat";
      return false;
  }
}

bool Perception::PublishState_6E2(unsigned char* msg)
{
  robot::canbusSpace::T_STATE_OUT_6E2 state_6E2;
  memset(&state_6E2, 0 ,sizeof(state_6E2));
  memcpy(&state_6E2, msg, CAN_MSG_LEN);
  if(state_6E2.iClbtStrAng0 != s_state_6E2.iClbtStrAng0 || state_6E2.iClbtStrAngLftMaxH != s_state_6E2.iClbtStrAngLftMaxH ||
          state_6E2.iClbtStrAngLftMaxL != s_state_6E2.iClbtStrAngLftMaxL || state_6E2.iClbtStrAngRtMax != s_state_6E2.iClbtStrAngRtMax)
  {

      memcpy(&s_state_6E2, msg, CAN_MSG_LEN);

      robot::canbusSpace::T_STR_LRN_PSNG strLrnPsng;
      robot::canbusSpace::T_STATE_OUT_6E2_UNION tStateOut6E2Union;
      memset(&tStateOut6E2Union,0,sizeof(robot::canbusSpace::T_STATE_OUT_6E2_UNION));
      tStateOut6E2Union.tStateOut6E2Src.iClbtStrAngLftMaxL = state_6E2.iClbtStrAngLftMaxL;
      tStateOut6E2Union.tStateOut6E2Src.iClbtStrAngLftMaxH = state_6E2.iClbtStrAngLftMaxH;
      strLrnPsng.iClbtStrAng0 = state_6E2.iClbtStrAng0;
      strLrnPsng.iClbtStrAngLftMax = tStateOut6E2Union.tStateOut6E2Dest.iClbtStrAngLftMax;
      strLrnPsng.iClbtStrAngRtMax = state_6E2.iClbtStrAngRtMax;

        strLrnPos_msg.ClbtStrAng0 = strLrnPsng.iClbtStrAng0 * 0.01;
        strLrnPos_msg.ClbtStrAngLftMax =  strLrnPsng.iClbtStrAngLftMax * 0.01;
        strLrnPos_msg.ClbtStrAngRtMax = strLrnPsng.iClbtStrAngRtMax * 0.01;

      return true;
  }
  else
  {
  //    LOG(INFO) << Name() << " PublishState_6E2 data is repeat";
      return false;
  }
}

bool Perception::PublishState_6E3(unsigned char *msg)
{
  robot::canbusSpace::T_STATE_OUT_6E3 state_6E3;
  memset(&state_6E3, 0 ,sizeof(state_6E3));
  memcpy(&state_6E3, msg, CAN_MSG_LEN);
  if(state_6E3.iDXMotPst != s_state_6E3.iDXMotPst || state_6E3.iNXMotPst != s_state_6E3.iNXMotPst ||
     state_6E3.iNYMotPst!= s_state_6E3.iNYMotPst || state_6E3.iPXMotPst != s_state_6E3.iPXMotPst ||
     state_6E3.iPYMotPst != s_state_6E3.iPYMotPst || state_6E3.iRXMotPst != s_state_6E3.iRXMotPst ||
     state_6E3.iRYMotPstH != s_state_6E3.iRYMotPstH || state_6E3.iRYMotPstL != s_state_6E3.iRYMotPstL)
  {

      memcpy(&s_state_6E3, msg, CAN_MSG_LEN);

      robot::canbusSpace::T_SFT_LRN_PST_1 sftLrnPsng1;


      robot::canbusSpace::T_STATE_OUT_6E3_UNION tStateOut6E3Union;
      memset(&tStateOut6E3Union,0,sizeof(robot::canbusSpace::T_STATE_OUT_6E3_UNION));
      tStateOut6E3Union.tStateOut6E3Src.iRYMotPstL = state_6E3.iRYMotPstL;
      tStateOut6E3Union.tStateOut6E3Src.iRYMotPstH = state_6E3.iRYMotPstH;
      sftLrnPsng1.iDXMotPst = state_6E3.iDXMotPst;
      sftLrnPsng1.iNXMotPst = state_6E3.iNXMotPst;
      sftLrnPsng1.iNYMotPst = state_6E3.iNYMotPst;
      sftLrnPsng1.iPXMotPst = state_6E3.iPXMotPst;
      sftLrnPsng1.iPYMotPst = state_6E3.iPYMotPst;
      sftLrnPsng1.iRXMotPst = state_6E3.iRXMotPst;
      sftLrnPsng1.iRYMotPst = tStateOut6E3Union.tStateOut6E3Dest.iRYMotPst;


        sftmsg.PXMotPst = sftLrnPsng1.iPXMotPst;
        sftmsg.PYMotPst = sftLrnPsng1.iPYMotPst;
        sftmsg.RXMotPst = sftLrnPsng1.iRXMotPst;
        sftmsg.RYMotPst = sftLrnPsng1.iRYMotPst;
        sftmsg.NXMotPst = sftLrnPsng1.iNXMotPst;
        sftmsg.NYMotPst = sftLrnPsng1.iNYMotPst;
        sftmsg.DXMotPst = sftLrnPsng1.iDXMotPst;

      return true;
  }
  else
  {
  //    LOG(INFO) << Name() << " PublishState_6E3 data is repeat";
      return false;
  }
}

bool Perception::PublishState_6EF(unsigned char *msg)
{
  robot::canbusSpace::T_STATE_OUT_6EF state_6EF;
  memset(&state_6EF, 0 ,sizeof(state_6EF));
  memcpy(&state_6EF, msg, CAN_MSG_LEN);
  if(state_6EF.iDownXMotPst != s_state_6EF.iDownXMotPst || state_6EF.iDownYMotPst != s_state_6EF.iDownYMotPst ||
     state_6EF.iDYMotPst!= s_state_6EF.iDYMotPst || state_6EF.iMXMotPst != s_state_6EF.iMXMotPst ||
     state_6EF.iMYMotPst != s_state_6EF.iMYMotPst || state_6EF.iUpXMotPstH != s_state_6EF.iUpXMotPstH ||
     state_6EF.iUpXMotPstL != s_state_6EF.iUpXMotPstL || state_6EF.iUpYMotPst != s_state_6EF.iUpYMotPst)
  {

      memcpy(&s_state_6EF, msg, CAN_MSG_LEN);

      robot::canbusSpace::T_SFT_LRN_PST_2 sftLrnPsng2;


      robot::canbusSpace::T_STATE_OUT_6EF_UNION tStateOut6EFUnion;
      memset(&tStateOut6EFUnion,0,sizeof(robot::canbusSpace::T_STATE_OUT_6EF_UNION));
      tStateOut6EFUnion.tStateOut6EFSrc.iUpXMotPstL = state_6EF.iUpXMotPstL;
      tStateOut6EFUnion.tStateOut6EFSrc.iUpXMotPstH = state_6EF.iUpXMotPstH;
      sftLrnPsng2.iDownXMotPst = state_6EF.iDownXMotPst;
      sftLrnPsng2.iDownYMotPst =state_6EF.iDownYMotPst;
      sftLrnPsng2.iDYMotPst = state_6EF.iDYMotPst;
      sftLrnPsng2.iMXMotPst = state_6EF.iMXMotPst;
      sftLrnPsng2.iMYMotPst = state_6EF.iMYMotPst;
      sftLrnPsng2.iUpXMotPst = tStateOut6EFUnion.tStateOut6EFDest.iUpXMotPst;
      sftLrnPsng2.iUpYMotPst = state_6EF.iUpYMotPst;

        sft2msg.DownXMotPst = sftLrnPsng2.iDownXMotPst;
        sft2msg.DownYMotPst = sftLrnPsng2.iDownYMotPst;
        sft2msg.DYMotPst = sftLrnPsng2.iDYMotPst;
        sft2msg.MXMotPst = sftLrnPsng2.iMXMotPst;
        sft2msg.MYMotPst = sftLrnPsng2.iMYMotPst;
        sft2msg.UpXMotPst = sftLrnPsng2.iUpXMotPst;
        sft2msg.UpYMotPst = sftLrnPsng2.iUpYMotPst;

      return true;
  }
  else
  {
   //   LOG(INFO) << Name() << " PublishState_6EF data is repeat";
      return false;
  }
}

bool Perception::PublishState_6EB(unsigned char *msg)
{
  robot::canbusSpace::T_STATE_OUT_6EB state_6EB;
  memset(&state_6EB, 0 ,sizeof(state_6EB));
  memcpy(&state_6EB, msg, CAN_MSG_LEN);
  if(state_6EB.uiAccMotCur != s_state_6EB.uiAccMotCur || state_6EB.uiBrkMotCur != s_state_6EB.uiBrkMotCur ||
     state_6EB.uiClchMotCur != s_state_6EB.uiClchMotCur || state_6EB.uiRbtRunTime != s_state_6EB.uiRbtRunTime ||
     state_6EB.uiStrMotCur != s_state_6EB.uiStrMotCur || state_6EB.uiXSftMotCur != s_state_6EB.uiXSftMotCur ||
     state_6EB.uiYSftMotCur != s_state_6EB.uiYSftMotCur)
  {

      memcpy(&s_state_6EB, msg, CAN_MSG_LEN);

      si1_msg.AccMotCur = state_6EB.uiAccMotCur * 0.1;
      si1_msg.BrkMotCur =  state_6EB.uiBrkMotCur * 0.1;
      si1_msg.ClchMotCur = state_6EB.uiClchMotCur * 0.1;
      si1_msg.RbtRunTime = state_6EB.uiRbtRunTime;
      si1_msg.StrMotCur = state_6EB.uiStrMotCur * 0.1;
      si1_msg.XSftMotCur = state_6EB.uiXSftMotCur * 0.1;
      si1_msg.YSftMotCur = state_6EB.uiYSftMotCur * 0.1;


      return true;
  }
  else
  {
  //    LOG(INFO) << Name() << " PublishState_6EB data is repeat";
      return false;
  }

}

bool Perception::PublishState_6D0(unsigned char *msg)
{
  robot::canbusSpace::T_STATE_OUT_6D0 state_6D0;
  memset(&state_6D0, 0 ,sizeof(state_6D0));
  memcpy(&state_6D0, msg, CAN_MSG_LEN);
  if(state_6D0.iCtrTmp != s_state_6D0.iCtrTmp || state_6D0.uiAccAbnStat != s_state_6D0.uiAccAbnStat ||
     state_6D0.uiBrkAbnStat != s_state_6D0.uiBrkAbnStat || state_6D0.uiClchAbnStat != s_state_6D0.uiClchAbnStat ||
     state_6D0.uiDrvCtrCanState != s_state_6D0.uiDrvCtrCanState || state_6D0.uiIOCtrCommunicateState != s_state_6D0.uiIOCtrCommunicateState ||
     state_6D0.uiRbtSlfChkStat != s_state_6D0.uiRbtSlfChkStat || state_6D0.uiRCUHdwAbnStat != s_state_6D0.uiRCUHdwAbnStat ||
     state_6D0.uiRCUSftwAbnStat != s_state_6D0.uiRCUSftwAbnStat || state_6D0.uiStrAbnStat != s_state_6D0.uiStrAbnStat ||
     state_6D0.uiSysAbnStat != s_state_6D0.uiSysAbnStat || state_6D0.uiXSftAbnStat != s_state_6D0.uiXSftAbnStat ||
     state_6D0.uiYSftAbnStat != s_state_6D0.uiYSftAbnStat)
  {

      memcpy(&s_state_6D0, msg, CAN_MSG_LEN);

      sm_msg.AccAbnStat = state_6D0.uiAccAbnStat;
      sm_msg.BrkAbnStat = state_6D0.uiBrkAbnStat;
      sm_msg.ClchAbnStat = state_6D0.uiClchAbnStat;
      sm_msg.CtrTmp = state_6D0.iCtrTmp;
      sm_msg.DrvCtrCanState = state_6D0.uiDrvCtrCanState;
      sm_msg.IOCtrCommunicateState = state_6D0.uiIOCtrCommunicateState;
      sm_msg.RbtSlfChkStat = state_6D0.uiRbtSlfChkStat;
      sm_msg.RCUHdwAbnStat = state_6D0.uiRCUHdwAbnStat;
      sm_msg.RCUSftwAbnStat = state_6D0.uiRCUSftwAbnStat;
      sm_msg.StrAbnStat = state_6D0.uiStrAbnStat;
      sm_msg.SysAbnStat = state_6D0.uiSysAbnStat;
      sm_msg.XSftAbnStat = state_6D0.uiXSftAbnStat;
      sm_msg.YSftAbnStat = state_6D0.uiYSftAbnStat;

      return true;
  }
  else
  {
   //   LOG(INFO) << Name() << " PublishState_6D0 data is repeat";
      return false;
  }
}

bool Perception::PublishState_6F5(unsigned char* msg)
{
//    int copySize;
//    if(msg.size() < sizeof(s_state_6F5))
//        copySize = msg->data.size();
//    else
//        copySize = sizeof(s_state_6F5);

    robot::canbusSpace::T_STATE_OUT_6F5 state_6F5;
    memset(&state_6F5, 0 ,sizeof(state_6F5));
    memcpy(&state_6F5, msg, CAN_MSG_LEN);

    if(state_6F5.iZSftMotAng != s_state_6F5.iZSftMotAng || state_6F5.iZSftMotSpd != s_state_6F5.iZSftMotSpd ||
            state_6F5.iZSftMotTrq != s_state_6F5.iZSftMotTrq ||
            state_6F5.uiZSftArmCtrAlive != s_state_6F5.uiZSftArmCtrAlive || state_6F5.uiZSftMotClbr != s_state_6F5.uiZSftMotClbr ||
            state_6F5.uiZSftMotCur != s_state_6F5.uiZSftMotCur)
    {

        memcpy(&s_state_6F5, msg, CAN_MSG_LEN);

        zsft_msg.ZSftArmCtrAlive = state_6F5.uiZSftArmCtrAlive;
        zsft_msg.ZSftMotAng =  state_6F5.iZSftMotAng;
        zsft_msg.ZSftMotClbr = state_6F5.uiZSftMotClbr;
        zsft_msg.ZSftMotCur = state_6F5.uiZSftMotCur;
        zsft_msg.ZSftMotSpd = state_6F5.iZSftMotSpd;
        zsft_msg.ZSftMotTrq = state_6F5.iZSftMotTrq;

        return true;
    }
    else
    {
 //       LOG(INFO) << Name() << " PublishState_6F5 data is repeat";
        return false;
    }
}

void Perception::OnTimer(const ros::TimerEvent &)
{
  try
  {
    if(bus_pub_6E5)
    {
       bus_pub_6E5.publish(aes_msg);
    }
    else
    {
        LOG(ERROR) << Name() << " PublishState_6E5 publisher is unvalid";
    }

    if(bus_pub_6E4)
    {
       bus_pub_6E4.publish(power_msg);
    }
    else
    {
        LOG(ERROR) << Name() << " PublishState_6E4 publisher is unvalid";
    }

    if(bus_pub_6EE)
    {
        bus_pub_6EE.publish(sl_msg);
    }
    else
    {
        LOG(ERROR) << Name() << " PublishState_6EE publisher is unvalid";
    }


    if(bus_pub_6ED)
    {
        bus_pub_6ED.publish(si2_msg);
    }
    else
    {
        LOG(ERROR) << Name() << " PublishState_6ED publisher is unvalid";
    }

    if(bus_pub_6D1)
    {
        bus_pub_6D1.publish(se_msg);
    }
    else
    {
        LOG(ERROR) << Name() << " PublishState_6D1 publisher is unvalid";
    }

    if(bus_pub_6D2)
    {
       bus_pub_6D2.publish(cec_msg22);
    }
    else
    {
        LOG(ERROR) << Name() << " PublishState_6D2 publisher  is unvalid";
    }

    if(bus_pub_6D3)
    {
       bus_pub_6D3.publish(cec_msg23);
    }
    else
    {
        LOG(ERROR) << Name() << " PublishState_6D3 publisher  is unvalid";
    }

    if(bus_pub_6D4)
    {
       bus_pub_6D4.publish(cec_msg4);
    }
    else
    {
        LOG(ERROR) << Name() << " PublishState_6D4 publisher  is unvalid";
    }

    if(bus_pub_6D5)
    {
       bus_pub_6D5.publish(cec_msg5);
    }
    else
    {
        LOG(ERROR) << Name() << " PublishState_6D5 publisher  is unvalid";
    }

    if(bus_pub_6D6)
    {
       bus_pub_6D6.publish(cec_msg6);
    }
    else
    {
        LOG(ERROR) << Name() << " PublishState_6D6 publisher  is unvalid";
    }

    if(bus_pub_2E0)
    {
        bus_pub_2E0.publish(ei_msg);
    }
    else
    {
        LOG(ERROR) << Name() << " PublishState_2E0 publisher is unvalid";
    }

    if(bus_pub_2E1)
    {
        bus_pub_2E1.publish(ee_msg);
    }
    else
    {
        LOG(ERROR) << Name() << " PublishState_2E1 publisher is unvalid";
    }

    if(bus_pub_6E1)
    {
        bus_pub_6E1.publish(pedLrnPos_msg);
    }
    else
    {
        LOG(ERROR) << Name() << " PublishState_6E1 publisher is unvalid";
    }

    if(bus_pub_6E2)
    {
        bus_pub_6E2.publish(strLrnPos_msg);
    }
    else
    {
        LOG(ERROR) << Name() << " PublishState_6E2 publisher is unvalid";
    }

    if(bus_pub_6E3)
    {
       bus_pub_6E3.publish(sftmsg);
    }
    else
    {
        LOG(ERROR) << Name() << " PublishState_6E3 publisher is unvalid";
    }

    if(bus_pub_6EF)
    {
        bus_pub_6EF.publish(sft2msg);
    }
    else
    {
        LOG(ERROR) << Name() << " PublishState_6EF publisher is unvalid";
    }

    if(bus_pub_6EB)
    {
        bus_pub_6EB.publish(si1_msg);
    }
    else
    {
        LOG(ERROR) << Name() << " PublishState_6EB publisher is unvalid";
    }

    if(bus_pub_6D0)
    {
      bus_pub_6D0.publish(sm_msg);
    }
    else
    {
        LOG(ERROR) << Name() << " PublishState_6D0 publisher is unvalid";
    }

    if(bus_pub_6F5)
    {
       bus_pub_6F5.publish(zsft_msg);
    }
    else
    {
        LOG(ERROR) << Name() << " PublishState_6F5 publisher is unvalid";
    }

  }
  catch (ros::Exception ex)
  {
       LOG(ERROR)<<Name()<< " OnTimer error. "<< ex.what();
  }
}

}  // namespace perception
}  // namespace robot
